! =================================================================================================================================
! MODULE       : stomate_io
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2011)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        This module reads and writes STOMATE variables in restart files.
!!
!!\n DESCRIPTION:  Not all variables saved in the start files are absolutely necessary.
!!                 However, Sechiba's and Stomate's PFTs are not necessarily identical, 
!!                 and for that case this information needs to be saved.
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	: None
!!
!! SVN          :
!! $HeadURL$
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================

MODULE stomate_io

  USE stomate_data
  USE constantes
  USE structures
  USE constantes_soil
  USE mod_orchidee_para
  USE ioipsl_para
  USE interpol_help
  USE function_library,   ONLY : wood_to_qmheight, biomass_to_lai 
  USE grid

  IMPLICIT NONE

  PRIVATE
  PUBLIC readrestart, writerestart, readbc, get_reftemp_clear

  LOGICAL, SAVE                                :: firstcall = .TRUE. !! first call?
!$OMP THREADPRIVATE(firstcall)

  REAL(r_std), ALLOCATABLE, DIMENSION(:), SAVE :: trefe              !! reference temperature (K)
!$OMP THREADPRIVATE(trefe)


CONTAINS

!! ================================================================================================================================
!! SUBROUTINE   : readrestart
!!
!>\BRIEF         read start file
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE readrestart (npts, index, lalo, resolution, neighbours, contfrac, &
       day_counter, dt_days, date, ind, &
       adapted, regenerate, moiavail_daily, gdd_init_date, &
       litterhum_daily, t2m_daily, t2m_min_daily, tsurf_daily, &
       tsoil_daily, soilhum_daily, precip_daily, gpp_daily, &
       npp_daily, turnover_daily, moiavail_month, moiavail_week, &
       moiavail_growingseason, t2m_longterm, tlong_ref, t2m_month, &
       t2m_week, tsoil_month, soilhum_month, fireindex, &
       firelitter, maxmoiavail_lastyear, maxmoiavail_thisyear, minmoiavail_lastyear, &
       minmoiavail_thisyear, maxgppweek_lastyear, maxgppweek_thisyear, gdd0_lastyear, &
       gdd0_thisyear, precip_lastyear, precip_thisyear, gdd_m5_dormance,  &
       gdd_from_growthinit, gdd_midwinter, ncd_dormance, ngd_minus5, &
       PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
       maxfpc_lastyear, maxfpc_thisyear, turnover_longterm, gpp_week, &
       biomass, resp_maint_part, leaf_age, leaf_frac, &
       senescence, senescence_days, when_growthinit, age, &
       resp_hetero, resp_maint, resp_growth, co2_fire, &
       co2_to_bm_dgvm, veget_lastlight, everywhere, need_adjacent, &
       RIP_time, time_lowgpp, time_hum_min, hum_min_dormance, &
       litterpart, litter, dead_leaves, carbon, &
       black_carbon, lignin_struc, lignin_wood, turnover_time, &
       prod_s, prod_m, prod_l, flux_s, flux_m, flux_l, &
       cflux_prod_s, cflux_prod_m, cflux_prod_l, bm_to_litter, &
       carb_mass_total, &
       age_stand, rotation_n, last_cut, &
       forest_managed, forest_managed_lastyear, global_years, &
       ok_equilibrium, nbp_accu, nbp_flux, MatrixV, &
       VectorU, previous_stock, current_stock, rue_longterm, &
       fpc_max, circ_class_n, circ_class_biomass, &
       KF, mai, &
       pai, previous_wood_volume, mai_count, coppice_dens, &
       vir_moiavail_growingseason, lai_per_level, laieff_fit, &
       temp_growth, wstress_season, wstress_month, vir_wstress_fac, &
       c_buried, k_latosa_adapt, species_change_map, fm_change_map, &
       lpft_replant)

    IMPLICIT NONE

    !! 0. Parameters and variables declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                                          :: npts                    !! Domain size
    INTEGER(i_std), DIMENSION(npts), INTENT(in)                         :: index                   !! Indices of the points on the map
    REAL(r_std), DIMENSION(npts,2), INTENT(in)                          :: lalo                    !! Geogr. coordinates 
    !! (latitude,longitude) (degrees)
    REAL(r_std), DIMENSION(npts,2), INTENT(in)                          :: resolution              !! size in x an y of the grid (m)
    INTEGER(i_std),DIMENSION (npts,8), INTENT(in)                       :: neighbours              !! Neighbouring land grid cell
    REAL(r_std),DIMENSION (npts), INTENT(in)                            :: contfrac                !! Fraction of land in each grid box

    !! 0.2 Output variables

    REAL(r_std), INTENT(out)                                            :: day_counter             !! counts time until next STOMATE time step
    REAL(r_std), INTENT(out)                                            :: dt_days                 !! time step of STOMATE in days
    INTEGER(i_std), INTENT(out)                                         :: date                    !! date (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: ind                     !! density of individuals
    !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: adapted                 !! Winter too cold? between 0 and 1
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: regenerate              !! Winter sufficiently cold? between 0 and 1
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: moiavail_daily          !! daily moisture availability
    REAL(r_std), DIMENSION(npts,4), INTENT(out)                         :: gdd_init_date           !! date for beginning of gdd count
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: litterhum_daily         !! daily litter humidity
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: t2m_daily               !! daily 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: t2m_min_daily           !! daily minimum 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: tsurf_daily             !! daily surface temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(out)                      :: tsoil_daily             !! daily soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(out)                      :: soilhum_daily           !! daily soil humidity
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: precip_daily            !! daily precipitations (mm/day) (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: gpp_daily               !! daily gross primary productivity
                                                                                                   !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: npp_daily               !! daily net primary productivity 
                                                                                                   !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out)      :: turnover_daily          !! daily turnover rates
                                                                                                   !! @tex $(gC m^{-2} dt_slow^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: moiavail_month          !! "monthly" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: moiavail_week           !! "weekly" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: moiavail_growingseason  !! "growingseason" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: vir_moiavail_growingseason !! Virtual "growingseason" moisture availability
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: t2m_longterm            !! "long term" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: t2m_month               !! "monthly" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: t2m_week                !! "weekly" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(out)                      :: tsoil_month             !! "monthly" soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(out)                      :: soilhum_month           !! "monthly" soil humidity
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: fireindex               !! Probability of fire
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: firelitter              !! Longer term total litter above 
    !! the ground
    !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: maxmoiavail_lastyear    !! last year's maximum 
    !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: maxmoiavail_thisyear    !! this year's maximum 
    !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: minmoiavail_lastyear    !! last year's minimum 
    !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: minmoiavail_thisyear    !! this year's minimum 
    !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: maxgppweek_lastyear     !! last year's maximum weekly GPP
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: maxgppweek_thisyear     !! this year's maximum weekly GPP
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: gdd0_lastyear           !! last year's annual GDD0
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: gdd0_thisyear           !! this year's annual GDD0
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: precip_lastyear         !! last year's annual precipitation
                                                                                                   !! (mm/year)
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: precip_thisyear         !! this year's annual precipitation 
                                                                                                   !! (mm/year)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: gdd_m5_dormance         !! growing degree days, threshold 
                                                                                                   !! -5 deg C (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: gdd_from_growthinit     !! growing degree days, 
                                                                                                   !! from begin of season
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: gdd_midwinter           !! growing degree days since 
                                                                                                   !! midwinter (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: ncd_dormance            !! number of chilling days since 
                                                                                                   !! leaves were lost (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: ngd_minus5              !! number of growing days, threshold 
                                                                                                   !! -5 deg C (for phenology)
    LOGICAL, DIMENSION(npts,nvm), INTENT(out)                           :: PFTpresent              !! PFT exists (equivalent to 
                                                                                                   !! fpc_max > 0 for natural PFTs)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: npp_longterm            !! "long term" net primary productivity 
                                                                                                   !! (gC/m**2/year)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: lm_lastyearmax          !! last year's maximum leaf mass, 
                                                                                                   !! for each PFT (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: lm_thisyearmax          !! this year's maximum leaf mass, 
                                                                                                   !! for each PFT (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: maxfpc_lastyear         !! last year's maximum fpc for each 
                                                                                                   !! natural PFT, on ground
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: maxfpc_thisyear         !! this year's maximum fpc for each 
                                                                                                   !! PFT, on *total* ground (see stomate_season)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out)      :: turnover_longterm       !! "long term" turnover rate (gC/m**2/year)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: gpp_week                !! "weekly" GPP (gC/day/(m**2 covered)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out)      :: biomass                 !! biomass (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(out)                :: resp_maint_part         !! maintenance resp (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(out)             :: leaf_age                !! leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(out)             :: leaf_frac               !! fraction of leaves in leaf age class
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(out)                 :: circ_class_n            !! Number of trees in each circumference classes
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), INTENT(out):: circ_class_biomass      !! Biomass components of the model tree within 
                                                                                                   !! each circumference classes (gC ind^-1)
    LOGICAL, DIMENSION(npts,nvm), INTENT(out)                           :: senescence              !! is the plant senescent ? 
                                                                                                   !! (only for deciduous trees - 
                                                                                                   !! carbohydrate reserve)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: senescence_days         !! Days since start of senescence (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: when_growthinit         !! how many days ago was the beginning
                                                                                                   !! of the growing season
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: age                     !! mean age (years)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: resp_hetero             !! heterotrophic respiration (gC/day/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: resp_maint              !! maintenance respiration (gC/day/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: resp_growth             !! growth respiration (gC/day/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: co2_fire                !! carbon emitted into the atmosphere
                                                                                                   !! by fire (living and dead biomass)
                                                                                                   !! (in gC/m**2/time step)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: co2_to_bm_dgvm          !! biomass uptaken 
                                                                                                   !! (gC/(m**2 of total ground)/day)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: veget_lastlight         !! vegetation fractions (on ground) 
                                                                                                   !! after last light competition
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: everywhere              !! is the PFT everywhere in the 
                                                                                                   !! grid box or very localized 
                                                                                                   !! (after its introduction)
    LOGICAL, DIMENSION(npts,nvm), INTENT(out)                           :: need_adjacent           !! in order for this PFT to be introduced, 
                                                                                                   !! does it have to be present in an 
                                                                                                   !! adjacent grid box?
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: RIP_time                !! How much time ago was the PFT eliminated
                                                                                                   !! for the last time (years) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: time_lowgpp             !! Duration of dormance (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: time_hum_min            !! time elapsed since strongest moisture
                                                                                                   !! availability (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: hum_min_dormance        !! minimum moisture during dormance
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(out)                 :: litterpart              !! fraction of litter above the ground
                                                                                                   !! belonging to different PFTs separated 
                                                                                                   !! for natural and agricultural PFTs.
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(out) :: litter                  !! metabolic and structural litter, 
                                                                                                   !! natural and agricultural, above and 
                                                                                                   !! below ground (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(out)                 :: dead_leaves             !! dead leaves on ground, per PFT, 
                                                                                                   !! metabolic and structural, 
                                                                                                   !! in gC/(m**2 of ground)
    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(out)                 :: carbon                  !! carbon pool: active, slow, or passive,
                                                                                                   !! (gC/m**2)
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: black_carbon            !! black carbon on the ground 
                                                                                                   !! (gC/(m**2 of total ground))
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(out)                 :: lignin_struc            !! ratio Lignine/Carbon in structural litter,
                                                                                                   !! above and below ground, 
                                                                                                   !! (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(out)                 :: lignin_wood             !! ratio Lignine/Carbon in woody litter,
                                                                                                   !! above and below ground, 
                                                                                                   !! (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(out)                :: turnover_time           !!
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: rue_longterm            !! longterm radiation use efficiency
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: fpc_max                 !! 

    INTEGER(i_std), DIMENSION(npts, nvm), INTENT(out)                   :: age_stand               !! Age of stand (years)
    INTEGER(i_std), DIMENSION(npts, nvm), INTENT(out)                   :: rotation_n              !! Rotation number (number of rotation 
                                                                                                   !! since pft is managed)
    INTEGER(i_std), DIMENSION(npts, nvm), INTENT(out)                   :: last_cut                !! Years since last thinning (years)
    INTEGER(i_std), DIMENSION (npts,nvm), INTENT(out)                   :: forest_managed_lastyear !! forest management flag for the previous year
    REAL(r_std), DIMENSION(npts,0:nshort), INTENT(out)                  :: prod_s                  !!
    REAL(r_std), DIMENSION(npts,0:nmedium), INTENT(out)                 :: prod_m                  !!
    REAL(r_std), DIMENSION(npts,0:nlong), INTENT(out)                   :: prod_l                  !!
    REAL(r_std), DIMENSION(npts,nshort), INTENT(out)                    :: flux_s                  !!
    REAL(r_std), DIMENSION(npts,nmedium), INTENT(out)                   :: flux_m                  !!
    REAL(r_std), DIMENSION(npts,nlong), INTENT(out)                     :: flux_l                  !!
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: cflux_prod_s            !!
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: cflux_prod_m            !!
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: cflux_prod_l            !!
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(out)      :: bm_to_litter            !!
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: carb_mass_total         !!
    INTEGER(i_std), INTENT(out)                                         :: global_years            !!
    LOGICAL, DIMENSION(npts), INTENT(out)                               :: ok_equilibrium          !!
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: nbp_accu                !! Accumulated Net Biospheric 
                                                                                                   !! Production over the year
    REAL(r_std), DIMENSION(npts), INTENT(out)                           :: nbp_flux                !! Net Biospheric Production over the year
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(out)       :: MatrixV                 !!
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out)               :: VectorU                 !!
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out)               :: previous_stock          !! 
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(out)               :: current_stock           !!  
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: KF                      !! Scaling factor to convert sapwood mass
                                                                                                   !! into leaf mass (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: k_latosa_adapt          !! Leaf to sapwood area adapted for water 
                                                                                                   !! stress. Adaptation takes place at the 
                                                                                                   !! end of the year (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: mai                     !! The mean annual increment
                                                                                                   !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: pai                     !! The period annual increment
                                                                                                   !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                       :: previous_wood_volume    !! The volume of the tree trunks
                                                                                                   !! in a stand for the previous year.
                                                                                                   !! @tex $(m**3 / m**2 )$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(out)                     :: mai_count               !! The number of times we've
                                                                                                   !! calculated the volume increment
                                                                                                   !! for a stand
    REAL(r_std), DIMENSION(npts,nvm),INTENT(out)                        :: coppice_dens            !! The density of a coppice at the first
                                                                                                   !! cutting.
                                                                                                   !! @tex $( 1 / m**2 )$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot),INTENT(out)            :: lai_per_level           !! The amount of LAI in each physical
                                                                                                   !! canopy level.
                                                                                                   !! @tex $( m**2 / m**2 )$ @endtex 
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(out)                     :: laieff_fit              !! Fitted parameters for the effective LAI
    REAL(r_std),DIMENSION(npts),INTENT(out)                             :: temp_growth             !! Growth temperature (°C) 
                                                                                                   !! Is equal to t2m_month
    REAL(r_std), DIMENSION(:,:), INTENT(out)                            :: vir_wstress_fac         !! Virtual water stress factor, based on hum_rel_daily
                                                                                                   !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(out)                            :: wstress_season          !! Water stress factor, based on hum_rel_daily
                                                                                                   !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(out)                            :: wstress_month           !! Water stress factor, based on hum_rel_daily
                                                                                                   !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(out)                            :: c_buried                !! Carbon that is buried when land cover changes
                                                                                                   !! to a nobio type.  
                                                                                                   !! @tex $( gC )$ @endtex 
    INTEGER(i_std), DIMENSION(:,:), INTENT(out)                         :: species_change_map      !! A map which gives the PFT number that each
                                                                                                   !! PFT will be replanted as in case of a clearcut.
                                                                                                   !! (1-nvm,unitless)
    INTEGER(i_std), DIMENSION(:,:), INTENT(out)                         :: fm_change_map           !! A map which gives the desired FM strategy when
                                                                                                   !! the PFT will be replanted after a clearcut.
                                                                                                   !! (1-nvm,unitless)
    LOGICAL, DIMENSION(:,:), INTENT(out)                                :: lpft_replant            !! Indicates if this PFT has either died this year
                                                                                                   !! or been clearcut/coppiced.  If it has, it is not
                                                                                                   !! replanted until the end of the year.

   !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts), INTENT(inout)                         :: tlong_ref               !! "long term" reference 2 meter temperatures (K)
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(inout)                  :: forest_managed          !! forest management flag

    !! 0.4 Local variables

    REAL(r_std)                                                         :: date_real               !! date, real
    REAL(r_std), DIMENSION(npts,nvm)                                    :: PFTpresent_real         !! PFT exists (equivalent to 
                                                                                                   !! fpc_max > 0 for natural PFTs), real
    REAL(r_std), DIMENSION(npts,nvm)                                    :: senescence_real         !! is the plant senescent ?
                                                                                                   !! (only for deciduous trees -
                                                                                                   !! carbohydrate reserve), real
    REAL(r_std), DIMENSION(npts,nvm)                                    :: need_adjacent_real      !! in order for this PFT to be introduced,
                                                                                                   !! does it have to be present in an 
                                                                                                   !! adjacent grid box? - real
    CHARACTER(LEN=80)                                                   :: var_name                !! To store variables names for I/O
    CHARACTER(LEN=10)                                                   :: part_str                !! string suffix indicating an index
    CHARACTER(LEN=10)                                                   :: part_str2               !! string suffix indicating an index
    CHARACTER(LEN=10)                                                   :: circ_str                !! string suffix indicating an index
    CHARACTER(LEN=3), DIMENSION(nlitt)                                  :: litter_str              !! string suffix indicating litter type
    CHARACTER(LEN=2), DIMENSION(nlevs)                                  :: level_str               !! string suffix indicating level
    REAL(r_std), DIMENSION(1)                                           :: xtmp                    !! temporary storage
    INTEGER(i_std)                                                      :: i,j,k,l,m,p,deb,&
                                                                           idia,ifm,icut           !! index
    REAL(r_std), DIMENSION(npts)                                        :: tref                    !! reference temperature (K)

    CHARACTER(LEN=2), DIMENSION(nelements)                              :: element_str             !! string suffix indicating element
    REAL(r_std), ALLOCATABLE, DIMENSION(:)	                        :: circ_ij                 !! circumference of idividual trees
    INTEGER(i_std)                                                      :: ntrees                  !! first or second part of self-thinning lines
    REAL(r_std), DIMENSION(npts,nvm)                                    :: temp_real               !! temporary real to allow restget 
                                                                                                   !! to work on multi-dimensional integers
    REAL(r_std)                                                         :: p_max, excedent         !! Where is the exponential distribution 
                                                                                                   !! truncated
    REAL(r_std), DIMENSION(20)                                          :: nb_trees_i              !! Number of trees in each twentith quantile
    REAL(r_std)                                                         :: lambda                  !! lambda of the exponential distribution 
                                                                                                   !! of initial diameters
    REAL(r_std)                                                         :: min_circ_init           !! Min initial circumference 
                                                                                                   !! of the exponential distribution
    REAL(r_std)                                                         :: max_circ_init           !! Max initial circumference
                                                                                                   !! of the exponential distribution
    REAL(r_std), DIMENSION(1)                                           :: temp_global_years       !!
    CHARACTER(LEN=7), DIMENSION(nbpools)                                :: pools_str               !!
    REAL(r_std), DIMENSION(npts)                                        :: ok_equilibrium_real     !!
    CHARACTER(LEN=80)                                                   :: cut_name                !! To store variables names for I/O
    CHARACTER(LEN=80)                                                   :: fm_name                 !! To store variables names for I/O
    REAL(r_std),DIMENSION(npts,nvm,nlevels_tot,nparams_laieff)          :: temp_array              !! To store structure values for I/O        
    INTEGER                                                             :: n,ilev,ipts,ivm         !! Indices
    REAL(r_std), DIMENSION(npts,nvm)                                    :: r_replant               !! Getting logical values from the restart
                                                                                                   !! is not possible, so this is a temporary
                                                                                                   !! array where 1.0 is TRUE.
    !_ ================================================================================================================================

    IF (bavard >= 3) WRITE(numout,*) 'Entering readrestart'
    !-
    ! 0 When the vegetation is dynamic,
    !   the long-term reference temperature is prognostic.
    !   In this case, it is read from the restart file.
    !   If the corresponding field does not exist in the restart file,
    !   read it from another file in order to initialize it correctly.
    !-
    CALL get_reftemp(npts, lalo, neighbours, resolution, contfrac, tref )
    !-
    ! 1 string definitions
    !-
    DO l=1,nlitt
       IF     (l == imetabolic) THEN
          litter_str(l) = 'met'
       ELSEIF (l == istructural) THEN
          litter_str(l) = 'str'
       ELSEIF (l == iwoody) THEN
          litter_str(l) = 'wod'
       ELSE
          CALL ipslerr_p (3,'stomate_io', &
               'Wrire restart - Define litter str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nlevs
       IF     (l == iabove) THEN
          level_str(l) = 'ab'
       ELSEIF (l == ibelow) THEN
          level_str(l) = 'be'
       ELSE
          CALL ipslerr_p (3,'stomate_io',&
               'Wrire restart - Define level str','','')
       ENDIF
    ENDDO

    pools_str(1:nbpools) =(/'str_ab ','str_be ','met_ab ',&
         'met_be ','wood_ab','wood_be', &
         'actif  ','slow   ','passif '/)

    !-
    DO l=1,nelements
       IF     (l == icarbon) THEN
          element_str(l) = ''
!!$       ELSEIF (l == initrogen) THEN
!!$          element_str(l) = '_n'
       ELSE
          CALL ipslerr_p (3,'stomate_io', &
               'Wrire restart - Define element str','','')
       ENDIF
    ENDDO
    !-
    ! 2 run control
    !-
    ! 2.1 day counter
    !-
    IF (is_root_prc) THEN
       var_name = 'day_counter'
       CALL restget (rest_id_stomate, var_name, 1   , 1     , 1, itime, &
            &                 .TRUE., xtmp)
       day_counter = xtmp(1)
       IF (day_counter == val_exp) day_counter = un
    ENDIF
    CALL bcast(day_counter)
    !-
    ! 2.2 time step of STOMATE in days
    !-
    IF (is_root_prc) THEN
       var_name = 'dt_days'
       CALL restget (rest_id_stomate, var_name, 1   , 1     , 1, itime, &
            &                 .TRUE., xtmp)
       dt_days = xtmp(1)
       IF (dt_days == val_exp) dt_days = un
    ENDIF
    CALL bcast(dt_days)
    !-
    ! 2.3 date
    !-
    IF (is_root_prc) THEN
       var_name = 'date'
       CALL restget (rest_id_stomate, var_name, 1   , 1     , 1, itime, &
            &                 .TRUE., xtmp)
       date_real = xtmp(1)
       IF (date_real == val_exp) date_real = zero
       date = NINT(date_real)
    ENDIF
    CALL bcast(date)
    !-
    ! 3 daily meteorological variables
    !-
    moiavail_daily(:,:) = val_exp
    var_name = 'moiavail_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_daily, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_daily(:,:) == val_exp)) moiavail_daily(:,:) = zero
    !-
    gdd_init_date(:,:) = val_exp
    var_name = 'gdd_init_date'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 4 , 1, itime, &
         &              .TRUE., gdd_init_date, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_init_date(:,1) == val_exp)) gdd_init_date(:,1) = 365.
    IF (ALL(gdd_init_date(:,3) == val_exp)) gdd_init_date(:,3) = zero
    IF (ALL(gdd_init_date(:,4) == val_exp)) gdd_init_date(:,4) = zero
    !-
    litterhum_daily(:) = val_exp
    var_name = 'litterhum_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., litterhum_daily, 'gather', nbp_glo, index_g)
    IF (ALL(litterhum_daily(:) == val_exp)) litterhum_daily(:) = zero
    !-
    t2m_daily(:) = val_exp
    var_name = 't2m_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., t2m_daily, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_daily(:) == val_exp)) t2m_daily(:) = zero
    !-
    t2m_min_daily(:) = val_exp
    var_name = 't2m_min_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., t2m_min_daily, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_min_daily(:) == val_exp)) t2m_min_daily(:) = large_value
    !-
    tsurf_daily(:) = val_exp
    var_name = 'tsurf_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., tsurf_daily, 'gather', nbp_glo, index_g)
    IF (ALL(tsurf_daily(:) == val_exp)) tsurf_daily(:) = tref(:)
    !-
    tsoil_daily(:,:) = val_exp
    var_name = 'tsoil_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &                .TRUE., tsoil_daily, 'gather', nbp_glo, index_g)
    IF (ALL(tsoil_daily(:,:) == val_exp)) tsoil_daily(:,:) = zero
    !-
    soilhum_daily(:,:) = val_exp
    var_name = 'soilhum_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &                .TRUE., soilhum_daily, 'gather', nbp_glo, index_g)
    IF (ALL(soilhum_daily(:,:) == val_exp)) soilhum_daily(:,:) = zero
    !-
    precip_daily(:) = val_exp
    var_name = 'precip_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., precip_daily, 'gather', nbp_glo, index_g)
    IF (ALL(precip_daily(:) == val_exp)) precip_daily(:) = zero
    !-
    ! 4 productivities
    !-
    gpp_daily(:,:) = val_exp
    var_name = 'gpp_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gpp_daily, 'gather', nbp_glo, index_g)
    IF (ALL(gpp_daily(:,:) == val_exp)) gpp_daily(:,:) = zero
    !-
    npp_daily(:,:) = val_exp
    var_name = 'npp_daily'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., npp_daily, 'gather', nbp_glo, index_g)
    IF (ALL(npp_daily(:,:) == val_exp)) npp_daily(:,:) = zero
    !-
    turnover_daily(:,:,:,:) = val_exp
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'turnover_daily_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
               &                .TRUE., turnover_daily(:,:,k,l), 'gather', nbp_glo, index_g)
          IF (ALL(turnover_daily(:,:,k,l) == val_exp)) &
               &       turnover_daily(:,:,k,l) = zero
       ENDDO
    END DO
    !-
    ! 5 monthly meteorological variables
    !-
    moiavail_month(:,:) = val_exp
    var_name = 'moiavail_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_month, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_month(:,:) == val_exp)) moiavail_month(:,:) = zero
    !-
    moiavail_week(:,:) = val_exp
    var_name = 'moiavail_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_week, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_week(:,:) == val_exp)) moiavail_week(:,:) = zero
    !-
    moiavail_growingseason(:,:) = val_exp
    var_name = 'moiavail_gs'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., moiavail_growingseason, 'gather', nbp_glo, index_g)
    IF (ALL(moiavail_growingseason(:,:) == val_exp)) moiavail_growingseason(:,:) = zero
    !-
    vir_moiavail_growingseason(:,:) = val_exp
    var_name = 'vir_moiavail_gs'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., vir_moiavail_growingseason, 'gather', nbp_glo, index_g)
    IF (ALL(vir_moiavail_growingseason(:,:) == val_exp)) vir_moiavail_growingseason(:,:) = zero
    !-
    t2m_longterm(:) = val_exp
    var_name = 't2m_longterm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_longterm, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_longterm(:) == val_exp)) t2m_longterm(:) = tref(:)
    !-
    ! the long-term reference temperature is a prognostic variable
    ! only in case the vegetation is dynamic
    !-
    IF (control%ok_dgvm) THEN
       tlong_ref(:) = val_exp
       var_name = 'tlong_ref'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
            &                .TRUE., tlong_ref, 'gather', nbp_glo, index_g)
       IF (ALL(tlong_ref(:) == val_exp)) tlong_ref(:) = tref(:)
    ENDIF
    !-
    t2m_month(:) = val_exp
    var_name = 't2m_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_month, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_month(:) == val_exp)) t2m_month(:) = tref(:)
    !-
    t2m_week(:) = val_exp
    var_name = 't2m_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., t2m_week, 'gather', nbp_glo, index_g)
    IF (ALL(t2m_week(:) == val_exp)) t2m_week(:) = tref(:)
    !-
    tsoil_month(:,:) = val_exp
    var_name = 'tsoil_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &              .TRUE., tsoil_month, 'gather', nbp_glo, index_g)

    IF (ALL(tsoil_month(:,:) == val_exp)) THEN
       DO l=1,nbdl
          tsoil_month(:,l) = tref(:)
       ENDDO
    ENDIF
    !-
    soilhum_month(:,:) = val_exp
    var_name = 'soilhum_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo,   nbdl, 1, itime, &
         &              .TRUE., soilhum_month, 'gather', nbp_glo, index_g)
    IF (ALL(soilhum_month(:,:) == val_exp)) soilhum_month(:,:) = zero
    !-
    temp_growth(:) = val_exp
    var_name = 'temp_growth'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., temp_growth, 'gather', nbp_glo, index_g)
    IF (ALL(temp_growth(:) == val_exp)) temp_growth(:) = zero
    !-
    wstress_season(:,:) = val_exp
    var_name = 'wstress_season'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., wstress_season, 'gather', nbp_glo, index_g)
    IF (ALL(wstress_season(:,:) == val_exp)) wstress_season(:,:) = un
    !-
    wstress_month(:,:) = val_exp
    var_name = 'wstress_month'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., wstress_month, 'gather', nbp_glo, index_g)
    IF (ALL(wstress_month(:,:) == val_exp)) wstress_month(:,:) = un
    !-
    vir_wstress_fac(:,:) = val_exp
    var_name = 'vir_wstress_fac'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., vir_wstress_fac, 'gather', nbp_glo, index_g)
    IF (ALL(vir_wstress_fac(:,:) == val_exp)) vir_wstress_fac(:,:) = un
    
    !-
    c_buried(:,:) = val_exp
    var_name = 'c_buried'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nnobio, 1, itime, &
         &                .TRUE., c_buried, 'gather', nbp_glo, index_g)
    IF (ALL(c_buried(:,:) == val_exp)) c_buried(:,:) = un

    !-
    ! 6 fire probability
    !-
    fireindex(:,:) = val_exp
    var_name = 'fireindex'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              .TRUE., fireindex, 'gather', nbp_glo, index_g)
    IF (ALL(fireindex(:,:) == val_exp)) fireindex(:,:) = zero
    !-
    firelitter(:,:) = val_exp
    var_name = 'firelitter'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              .TRUE., firelitter, 'gather', nbp_glo, index_g)
    IF (ALL(firelitter(:,:) == val_exp)) firelitter(:,:) = zero
    !-
    ! 7 maximum and minimum moisture availabilities for tropic phenology
    !-
    maxmoiavail_lastyear(:,:) = val_exp
    var_name = 'maxmoistr_last'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxmoiavail_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxmoiavail_lastyear(:,:) == val_exp)) &
         &     maxmoiavail_lastyear(:,:) = zero
    !-
    maxmoiavail_thisyear(:,:) = val_exp
    var_name = 'maxmoistr_this'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxmoiavail_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxmoiavail_thisyear(:,:) == val_exp)) &
         &     maxmoiavail_thisyear(:,:) = zero
    !-
    minmoiavail_lastyear(:,:) = val_exp
    var_name = 'minmoistr_last'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., minmoiavail_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(minmoiavail_lastyear(:,:) == val_exp)) &
         &     minmoiavail_lastyear(:,:) = un
    !-
    minmoiavail_thisyear(:,:) = val_exp
    var_name = 'minmoistr_this'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., minmoiavail_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL( minmoiavail_thisyear(:,:) == val_exp)) &
         &     minmoiavail_thisyear(:,:) = un
    !-
    ! 8 maximum "weekly" GPP
    !-
    maxgppweek_lastyear(:,:) = val_exp
    var_name = 'maxgppweek_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxgppweek_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxgppweek_lastyear(:,:) == val_exp)) &
         &     maxgppweek_lastyear(:,:) = zero
    !-
    maxgppweek_thisyear(:,:) = val_exp
    var_name = 'maxgppweek_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxgppweek_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxgppweek_thisyear(:,:) == val_exp)) &
         &     maxgppweek_thisyear(:,:) = zero
    !-
    ! 9 annual GDD0
    !-
    gdd0_thisyear(:) = val_exp
    var_name = 'gdd0_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., gdd0_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(gdd0_thisyear(:) == val_exp)) gdd0_thisyear(:) = zero
    !-
    gdd0_lastyear(:) = val_exp
    var_name = 'gdd0_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., gdd0_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(gdd0_lastyear(:) == val_exp)) gdd0_lastyear(:) = gdd_crit_estab
    !-
    ! 10 annual precipitation
    !-
    precip_thisyear(:) = val_exp
    var_name = 'precip_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., precip_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(precip_thisyear(:) == val_exp)) precip_thisyear(:) = zero
    !-
    precip_lastyear(:) = val_exp
    var_name = 'precip_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., precip_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(precip_lastyear(:) == val_exp)) &
         &     precip_lastyear(:) = precip_crit
    !-
    ! 11 derived "biometeorological" variables
    !-
    gdd_m5_dormance(:,:) = val_exp
    var_name = 'gdd_m5_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_m5_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_m5_dormance(:,:) == val_exp)) &
         &     gdd_m5_dormance(:,:) = undef
    !-
    gdd_from_growthinit(:,:) = val_exp
    var_name = 'gdd_from_growthinit'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_from_growthinit, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_from_growthinit(:,:) == val_exp)) &
         &     gdd_from_growthinit(:,:) = zero
    !-
    gdd_midwinter(:,:) = val_exp
    var_name = 'gdd_midwinter'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gdd_midwinter, 'gather', nbp_glo, index_g)
    IF (ALL(gdd_midwinter(:,:) == val_exp)) gdd_midwinter(:,:) = undef
    !-
    ncd_dormance(:,:) = val_exp
    var_name = 'ncd_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ncd_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(ncd_dormance(:,:) == val_exp)) ncd_dormance(:,:) = undef
    !-
    ngd_minus5(:,:) = val_exp
    var_name = 'ngd_minus5'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ngd_minus5, 'gather', nbp_glo, index_g)
    IF (ALL(ngd_minus5(:,:) == val_exp)) ngd_minus5(:,:) = zero
    !-
    time_lowgpp(:,:) = val_exp
    var_name = 'time_lowgpp'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., time_lowgpp, 'gather', nbp_glo, index_g)
    IF (ALL(time_lowgpp(:,:) == val_exp)) time_lowgpp(:,:) = zero
    !-
    time_hum_min(:,:) = val_exp
    var_name = 'time_hum_min'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., time_hum_min, 'gather', nbp_glo, index_g)
    IF (ALL(time_hum_min(:,:) == val_exp)) time_hum_min(:,:) = undef
    !-
    hum_min_dormance(:,:) = val_exp
    var_name = 'hum_min_dormance'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., hum_min_dormance, 'gather', nbp_glo, index_g)
    IF (ALL(hum_min_dormance(:,:) == val_exp)) &
         &     hum_min_dormance(:,:) = undef
    !-
    ! 12 Plant status
    !-
    PFTpresent_real(:,:) = val_exp
    var_name = 'PFTpresent'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., PFTpresent_real, 'gather', nbp_glo, index_g)
    IF (ALL(PFTpresent_real(:,:) == val_exp)) PFTpresent_real(:,:) = zero
    WHERE (PFTpresent_real(:,:) >= .5)
       PFTpresent = .TRUE.
    ELSEWHERE
       PFTpresent = .FALSE.
    ENDWHERE
    !-

    !+++++++++ CHECK MJM +++++++++++++++
    ! why is this done here?
!!$    IF ( control%forest_management ) THEN
!!$       
!!$       IF ( .NOT. ALLOCATED(circ_ij)) ALLOCATE(circ_ij(MAXVAL(nmaxtrees)))
!!$
!!$       !! initializing circ_ij and setting m to the smallest cir_ij greater than zero
!!$       p_max = 100./MAXVAL(nmaxtrees)
!!$       circ_ij(:) = zero
!!$       nb_trees_i(:) = 0
!!$
!!$       !! lambda of the exponential distribution of initial diameters
!!$       !! (Dhote 2003 : lambda = SQRT(2)/Dg and Dginit = 1 cm)
!!$       lambda = 2._r_std**0.5_r_std/pi
!!$
!!$       !! The distribution is truncated when P(X>max_circ_init)<100/maxval(nmaxtrees)
!!$       min_circ_init = pi
!!$       max_circ_init = 1./lambda*LOG(1/p_max)       
!!$
!!$       DO p = 1,20
!!$          nb_trees_i(p) = NINT(MAXVAL(nmaxtrees)*(EXP(-lambda*max_circ_init*(p-1)/20)-EXP(-lambda*max_circ_init*p/20)))
!!$       ENDDO
!!$       excedent = REAL(MAXVAL(nmaxtrees),r_std)-SUM(nb_trees_i)
!!$       nb_trees_i(:) = nb_trees_i(:) + FLOOR(excedent/20)
!!$       nb_trees_i(1) = nb_trees_i(1) + NINT(excedent) - 20*FLOOR(excedent/20)
!!$
!!$       DO p = 1,20
!!$          IF (nb_trees_i(p) > 0) THEN
!!$             deb = 0
!!$             IF (p > 1) deb = SUM(nb_trees_i(1:(p-1)))
!!$             DO k = 1,NINT(nb_trees_i(p))
!!$                circ_ij(deb+k) = min_circ_init + REAL(p-1)/20*max_circ_init + k*max_circ_init/(20*nb_trees_i(p))
!!$             ENDDO
!!$          ENDIF
!!$       ENDDO
!!$       circ_ij(:) = circ_ij(:)*0.01 !changed to meters
!!$       ntrees = COUNT(MASK = circ_ij .GT. zero)
!!$
!!$    END IF ! control%forest_management
    !++++++++++++++++++++

    !-
    ind(:,:) = val_exp
    var_name = 'ind'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., ind, 'gather', nbp_glo, index_g)
    IF (ALL(ind(:,:) == val_exp)) THEN 
       ind(:,:) = zero
       !+++++ CHECK +++++
       ! this makes it so managed forests don't go through prescribe, which seems incorrect
       ! maybe it was needed for the old allocation scheme?
       !       IF ( control%forest_management ) THEN
       !          DO j = 2,nvm
       !             WHERE ( forest_managed(:,j) > 0 ) ind(:,j) = REAL(ntrees)/10000.
       !          ENDDO
       !       END IF
       !+++++++++++++++++
    ENDIF
    !-
    adapted(:,:) = val_exp
    var_name = 'adapted'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., adapted, 'gather', nbp_glo, index_g)
    IF (ALL(adapted(:,:) == val_exp)) adapted(:,:) = zero
    !-
    regenerate(:,:) = val_exp
    var_name = 'regenerate'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., regenerate, 'gather', nbp_glo, index_g)
    IF (ALL(regenerate(:,:) == val_exp)) regenerate(:,:) = zero
    !-
    npp_longterm(:,:) = val_exp
    var_name = 'npp_longterm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., npp_longterm, 'gather', nbp_glo, index_g)
    IF (ALL(npp_longterm(:,:) == val_exp)) npp_longterm(:,:) = zero
    !-
    lm_lastyearmax(:,:) = val_exp
    var_name = 'lm_lastyearmax'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., lm_lastyearmax, 'gather', nbp_glo, index_g)
    IF (ALL(lm_lastyearmax(:,:) == val_exp)) lm_lastyearmax(:,:) = zero
    !-
    lm_thisyearmax(:,:) = val_exp
    var_name = 'lm_thisyearmax'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., lm_thisyearmax, 'gather', nbp_glo, index_g)
    IF (ALL(lm_thisyearmax(:,:) == val_exp)) lm_thisyearmax(:,:) = zero
    !-
    maxfpc_lastyear(:,:) = val_exp
    var_name = 'maxfpc_lastyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxfpc_lastyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxfpc_lastyear(:,:) == val_exp)) maxfpc_lastyear(:,:) = zero
    !-
    maxfpc_thisyear(:,:) = val_exp
    var_name = 'maxfpc_thisyear'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., maxfpc_thisyear, 'gather', nbp_glo, index_g)
    IF (ALL(maxfpc_thisyear(:,:) == val_exp)) maxfpc_thisyear(:,:) = zero
    !-
    turnover_time(:,:,:) = val_exp
    DO k = 1,nparts
       WRITE(part_str,'(I2)') k
       IF ( k < 10 ) part_str(1:1) = '0'
       var_name = 'turnover_time_'//part_str(1:LEN_TRIM(part_str))
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., turnover_time(:,:,k), 'gather', nbp_glo, index_g)
       IF ( ALL( turnover_time(:,:,k) == val_exp)) turnover_time(:,:,k) = 100.
    ENDDO
    !-
    turnover_longterm(:,:,:,:) = val_exp
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF ( k < 10 ) part_str(1:1) = '0'
          var_name = 'turnover_longterm_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
               &              .TRUE., turnover_longterm(:,:,k,l), 'gather', nbp_glo, index_g)
          IF (ALL(turnover_longterm(:,:,k,l) == val_exp)) &
               &       turnover_longterm(:,:,k,l) = zero
       ENDDO
    END DO
    !-
    gpp_week(:,:) = val_exp
    var_name = 'gpp_week'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., gpp_week, 'gather', nbp_glo, index_g)
    IF (ALL(gpp_week(:,:) == val_exp)) gpp_week(:,:) = zero
    !-
    biomass(:,:,:,:) = val_exp
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF ( k < 10 ) part_str(1:1) = '0'
          var_name = 'biomass_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
               &                   .TRUE., biomass(:,:,k,l), 'gather', nbp_glo, index_g)
          IF (ALL(biomass(:,:,k,l) == val_exp)) biomass(:,:,k,l) = zero
       ENDDO
    END DO
    !-
    resp_maint_part(:,:,:) = val_exp
    DO k=1,nparts
       WRITE(part_str,'(I2)') k
       IF ( k < 10 ) part_str(1:1) = '0'
       var_name = 'maint_resp_'//part_str(1:LEN_TRIM(part_str))
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &                   .TRUE., resp_maint_part(:,:,k), 'gather', nbp_glo, index_g)
       IF (ALL(resp_maint_part(:,:,k) == val_exp)) resp_maint_part(:,:,k) = zero
    ENDDO
    !-
    leaf_age(:,:,:) = val_exp
    DO m=1,nleafages
       WRITE (part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       var_name = 'leaf_age_'//part_str(1:LEN_TRIM(part_str))
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &                   .TRUE., leaf_age(:,:,m), 'gather', nbp_glo, index_g)
       IF (ALL(leaf_age(:,:,m) == val_exp)) leaf_age(:,:,m) = zero
    ENDDO
    !-
    leaf_frac(:,:,:) = val_exp
    DO m=1,nleafages
       WRITE(part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       var_name = 'leaf_frac_'//part_str(1:LEN_TRIM(part_str))
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &                  .TRUE., leaf_frac(:,:,m), 'gather', nbp_glo, index_g)
       IF (ALL(leaf_frac(:,:,m) == val_exp)) leaf_frac(:,:,m) = zero
    ENDDO
    !-
    circ_class_n(:,:,:) = val_exp
    DO m=1,ncirc
       WRITE(part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       var_name = 'circ_class_n_'//part_str(1:LEN_TRIM(part_str))
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &                  .TRUE., circ_class_n(:,:,m), 'gather', nbp_glo, index_g)
       IF (ALL(circ_class_n(:,:,m) == val_exp)) circ_class_n(:,:,m) = zero
    ENDDO
    !-
    circ_class_biomass(:,:,:,:,:) = val_exp
    DO m = 1,nelements
       DO l = 1,nparts
          DO k = 1,ncirc 
             WRITE(part_str,'(I2)') l
             IF ( l < 10 ) part_str(1:1) = '0'
             WRITE(circ_str, '(I2.2)') k
             var_name = 'cc_bio_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(m))//'_'//TRIM(circ_str)
             CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
                  &         .TRUE., circ_class_biomass(:,:,k,l,m), 'gather', nbp_glo, index_g)
             IF (ALL(circ_class_biomass(:,:,k,l,m) == val_exp)) circ_class_biomass(:,:,k,l,m) = zero
          ENDDO
       ENDDO
    ENDDO
    !-
    lai_per_level(:,:,:) = val_exp
    DO m=1,nlevels_tot
       WRITE(part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       var_name = 'lai_per_level_'//part_str(1:LEN_TRIM(part_str))
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &                  .TRUE., lai_per_level(:,:,m), 'gather', nbp_glo, index_g)
       IF (ALL(lai_per_level(:,:,m) == val_exp)) lai_per_level(:,:,m) = zero
    ENDDO
    !
    ! This is ugly, since the restart routines were not developed to deal with structures. 
    ! But all I have to do is copy some things to arrays.
    temp_array(:,:,:,:) = val_exp
    DO m=1,nlevels_tot
       WRITE(part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       DO n=1,nparams_laieff
          WRITE(part_str2,'(I2)') n
          IF ( n < 10 ) part_str2(1:1) = '0'

          var_name = 'laieff_fit__'//part_str(1:LEN_TRIM(part_str)) // '_' &
               // part_str2(1:LEN_TRIM(part_str2))
          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
               .TRUE., temp_array(:,:,m,n), 'gather', nbp_glo, index_g)
          IF (ALL(temp_array(:,:,m,n) == val_exp)) temp_array(:,:,m,n) = zero
       ENDDO
    ENDDO
    DO ipts=1,npts
       DO ivm=1,nvm
          DO ilev=1,nlevels_tot
             laieff_fit(ipts,ivm,ilev)%a=temp_array(ipts,ivm,ilev,1)
             laieff_fit(ipts,ivm,ilev)%b=temp_array(ipts,ivm,ilev,2)
             laieff_fit(ipts,ivm,ilev)%c=temp_array(ipts,ivm,ilev,3)
             laieff_fit(ipts,ivm,ilev)%d=temp_array(ipts,ivm,ilev,4)
             laieff_fit(ipts,ivm,ilev)%e=temp_array(ipts,ivm,ilev,5)
          ENDDO
       ENDDO
    ENDDO
    !-
    senescence_real(:,:) = val_exp
    var_name = 'senescence'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., senescence_real, 'gather', nbp_glo, index_g)
    IF (ALL(senescence_real(:,:) == val_exp)) senescence_real(:,:) = zero
    WHERE ( senescence_real(:,:) >= .5 )
       senescence = .TRUE.
    ELSEWHERE
       senescence = .FALSE.
    ENDWHERE
    !-
    senescence_days(:,:) = val_exp
    var_name = 'senescence_days'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &              .TRUE., senescence_days, 'gather', nbp_glo, index_g)
    IF (ALL(senescence_days(:,:) == val_exp)) &
         &     senescence_days(:,:) = zero
    !-
    when_growthinit(:,:) = val_exp
    var_name = 'when_growthinit'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., when_growthinit, 'gather', nbp_glo, index_g)
    ! Assume the last growing season started 8 months ago. This way deciduous
    ! pft can have leaves in year one although the phenology in the first year
    ! of a cold start may be off by up to 60 days or so.
    IF (ALL(when_growthinit(:,:) == val_exp)) &
         &     when_growthinit(:,:) = 240
    !-
    age(:,:) = val_exp
    var_name = 'age'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., age, 'gather', nbp_glo, index_g)
    IF (ALL(age(:,:) == val_exp)) age(:,:) = zero
    !-
    KF(:,:) = val_exp
    var_name = 'KF'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., KF, 'gather', nbp_glo, index_g)
    ! I don't want to set it equal to zero, since this is a problem if these
    ! values are not here!  Better it blows up later on
    !IF (ALL(KF(:,:) == val_exp)) KF(:,:) = zero

    k_latosa_adapt(:,:) = val_exp
    var_name = 'k_latosa_adapt'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., k_latosa_adapt, 'gather', nbp_glo, index_g)
    DO ivm = 1,nvm
       IF (ALL(k_latosa_adapt(:,ivm) == val_exp)) k_latosa_adapt(:,ivm) = k_latosa_min(ivm)
    ENDDO

    !-
    ! 13 CO2
    !-
    resp_hetero(:,:) = val_exp
    var_name = 'resp_hetero'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                .TRUE., resp_hetero, 'gather', nbp_glo, index_g)
    IF (ALL(resp_hetero(:,:) == val_exp)) resp_hetero(:,:) = zero
    !-
    resp_maint(:,:) = val_exp
    var_name = 'resp_maint'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., resp_maint, 'gather', nbp_glo, index_g)
    IF (ALL(resp_maint(:,:) == val_exp)) resp_maint(:,:) = zero
    !-
    resp_growth(:,:) = val_exp
    var_name = 'resp_growth'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., resp_growth, 'gather', nbp_glo, index_g)
    IF (ALL(resp_growth(:,:) == val_exp)) resp_growth(:,:) = zero
    !-
    co2_fire(:,:) = val_exp
    var_name = 'co2_fire'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
         &                .TRUE., co2_fire, 'gather', nbp_glo, index_g)
    IF (ALL(co2_fire(:,:) == val_exp)) co2_fire(:,:) = zero
    !-
    co2_to_bm_dgvm(:,:) = val_exp
    var_name = 'co2_to_bm_dgvm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm     , 1, itime, &
         &                .TRUE., co2_to_bm_dgvm, 'gather', nbp_glo, index_g)
    IF (ALL(co2_to_bm_dgvm(:,:) == val_exp)) co2_to_bm_dgvm(:,:) = zero
    !-
    ! 14 vegetation distribution after last light competition
    !-
    veget_lastlight(:,:) = val_exp
    var_name = 'veget_lastlight'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., veget_lastlight, 'gather', nbp_glo, index_g)
    IF (ALL(veget_lastlight(:,:) == val_exp)) veget_lastlight(:,:) = zero
    !-
    ! 15 establishment criteria
    !-
    everywhere(:,:) = val_exp
    var_name = 'everywhere'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., everywhere, 'gather', nbp_glo, index_g)
    IF (ALL(everywhere(:,:) == val_exp)) everywhere(:,:) = zero
    !-
    need_adjacent_real(:,:) = val_exp
    var_name = 'need_adjacent'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., need_adjacent_real, 'gather', nbp_glo, index_g)
    IF (ALL(need_adjacent_real(:,:) == val_exp)) &
         &     need_adjacent_real(:,:) = zero
    WHERE ( need_adjacent_real(:,:) >= .5 )
       need_adjacent = .TRUE.
    ELSEWHERE
       need_adjacent = .FALSE.
    ENDWHERE
    !-
    RIP_time(:,:) = val_exp
    var_name = 'RIP_time'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                .TRUE., RIP_time, 'gather', nbp_glo, index_g)
    IF (ALL(RIP_time(:,:) == val_exp)) RIP_time(:,:) = large_value
    !-
    ! 16 black carbon
    !-
    black_carbon(:) = val_exp
    var_name = 'black_carbon'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &                .TRUE., black_carbon, 'gather', nbp_glo, index_g)
    IF (ALL(black_carbon(:) == val_exp)) black_carbon(:) = zero
    !-
    ! 17 litter
    !-
    litterpart(:,:,:) = val_exp
    DO l=1,nlitt
       var_name = 'litterpart_'//litter_str(l)
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &                   .TRUE., litterpart(:,:,l), 'gather', nbp_glo, index_g)
       IF (ALL(litterpart(:,:,l) == val_exp)) litterpart(:,:,l) = zero
    ENDDO
    !-
    litter(:,:,:,:,:) = val_exp
    DO k = 1,nelements
       DO l = 1,nlevs
          DO m = 1,nvm
             WRITE (part_str, '(I2)') m
             IF (m<10) part_str(1:1)='0'
             var_name = 'litter_'//part_str(1:LEN_TRIM(part_str))//'_'//level_str(l)//TRIM(element_str(k))
             CALL restget_p (rest_id_stomate, var_name, nbp_glo, nlitt , 1, itime, &
                  &                     .TRUE., litter(:,:,m,l,k), 'gather', nbp_glo, index_g)
             IF (ALL(litter(:,:,m,l,k) == val_exp)) litter(:,:,m,l,k) = zero
          ENDDO
       ENDDO
    END DO
    !-
    dead_leaves(:,:,:) = val_exp
    DO l=1,nlitt
       var_name = 'dead_leaves_'//litter_str(l)
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &                   .TRUE., dead_leaves(:,:,l), 'gather', nbp_glo, index_g)
       IF (ALL(dead_leaves(:,:,l) == val_exp)) dead_leaves(:,:,l) = zero
    ENDDO
    !-
    carbon(:,:,:) = val_exp
    DO m=1,nvm
       WRITE (part_str, '(I2)') m
       IF (m<10) part_str(1:1)='0'
       var_name = 'carbon_'//part_str(1:LEN_TRIM(part_str))
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, ncarb , 1, itime, &
            &                   .TRUE., carbon(:,:,m), 'gather', nbp_glo, index_g)
       IF (ALL(carbon(:,:,m) == val_exp)) carbon(:,:,m) = zero
    ENDDO
    !-
    lignin_struc(:,:,:) = val_exp
    DO l=1,nlevs
       var_name = 'lignin_struc_'//level_str(l)
       CALL restget_p &
            &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &     .TRUE., lignin_struc(:,:,l), 'gather', nbp_glo, index_g)
       IF (ALL(lignin_struc(:,:,l) == val_exp)) lignin_struc(:,:,l) = zero
    ENDDO
    !-
    lignin_wood(:,:,:) = val_exp
    DO l=1,nlevs
       var_name = 'lignin_wood_'//level_str(l)
       CALL restget_p &
            &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &     .TRUE., lignin_wood(:,:,l), 'gather', nbp_glo, index_g)
       IF (ALL(lignin_wood(:,:,l) == val_exp)) lignin_wood(:,:,l) = zero
    ENDDO
    !-
    ! 18 land cover change
    !-
    prod_s(:,:) = val_exp
    var_name = 'prod_s'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nshort+1 , 1, itime, &
         &                .TRUE., prod_s, 'gather', nbp_glo, index_g)
    IF (ALL(prod_s(:,:) == val_exp)) prod_s(:,:) = zero

    prod_m(:,:) = val_exp
    var_name = 'prod_m'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nmedium+1, 1, itime, &
         &                .TRUE., prod_m, 'gather', nbp_glo, index_g)
    IF (ALL(prod_m(:,:) == val_exp)) prod_m(:,:) = zero

    prod_l(:,:) = val_exp
    var_name = 'prod_l'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nlong+1     , 1, itime, &
         &                .TRUE., prod_l, 'gather', nbp_glo, index_g)
    IF (ALL(prod_l(:,:) == val_exp)) prod_l(:,:) = zero

    flux_s(:,:) = val_exp
    var_name = 'flux_s'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nshort     , 1, itime, &
         &                .TRUE., flux_s, 'gather', nbp_glo, index_g)
    IF (ALL(flux_s(:,:) == val_exp)) flux_s(:,:) = zero

    flux_m(:,:) = val_exp
    var_name = 'flux_m'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nmedium     , 1, itime, &
         &                .TRUE., flux_m, 'gather', nbp_glo, index_g)
    IF (ALL(flux_m(:,:) == val_exp)) flux_m(:,:) = zero

    flux_l(:,:) = val_exp
    var_name = 'flux_l'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nlong     , 1, itime, &
         &                .TRUE., flux_l, 'gather', nbp_glo, index_g)
    IF (ALL(flux_l(:,:) == val_exp)) flux_l(:,:) = zero

    cflux_prod_s(:) = val_exp
    var_name = 'cflux_prod_s'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., cflux_prod_s, 'gather', nbp_glo, index_g)
    IF (ALL(cflux_prod_s(:) == val_exp)) cflux_prod_s(:) = zero

    cflux_prod_m(:) = val_exp
    var_name = 'cflux_prod_m'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., cflux_prod_m, 'gather', nbp_glo, index_g)
    IF (ALL(cflux_prod_m(:) == val_exp)) cflux_prod_m(:) = zero

    cflux_prod_l(:) = val_exp
    var_name = 'cflux_prod_l'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., cflux_prod_l, 'gather', nbp_glo, index_g)
    IF (ALL(cflux_prod_l(:) == val_exp)) cflux_prod_l(:) = zero

    rue_longterm(:,:) = val_exp
    var_name = 'rue_longterm'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., rue_longterm(:,:), 'gather', nbp_glo, index_g)
    IF (ALL(rue_longterm(:,:) == val_exp)) rue_longterm(:,:) = 1.

!!$    lai_target(:,:) = val_exp
!!$    var_name = 'lai_target'
!!$    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
!!$         &        .TRUE., lai_target(:,:), 'gather', nbp_glo, index_g)
!!$    IF (ALL(lai_target(:,:) == val_exp)) lai_target(:,:) = zero

    fpc_max(:,:) = val_exp
    var_name = 'fpc_max'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., fpc_max(:,:), 'gather', nbp_glo, index_g)
    IF (ALL(fpc_max(:,:) == val_exp)) fpc_max(:,:) = .25

    bm_to_litter(:,:,:,:) = val_exp
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF ( k < 10 ) part_str(1:1) = '0'
          var_name = 'bm_to_litter_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
               &                .TRUE., bm_to_litter(:,:,k,l), 'gather', nbp_glo, index_g)
          IF (ALL(bm_to_litter(:,:,k,l) == val_exp)) bm_to_litter(:,:,k,l) = zero
       ENDDO
    END DO

    carb_mass_total(:) = val_exp
    var_name = 'carb_mass_total'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
         &              .TRUE., carb_mass_total, 'gather', nbp_glo, index_g)
    IF (ALL(carb_mass_total(:) == val_exp)) carb_mass_total(:) = zero

    ! The routines don't like to store integer variables.  So instead
    ! we create a real array.  If the value of the real array is one,
    ! we assign a value of TRUE to the logical array.  Anything
    ! else is .FALSE.
    r_replant(:,:) = val_exp
    var_name = 'sc_map'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., r_replant(:,:), 'gather', nbp_glo, index_g)
    DO ipts=1,npts
       DO ivm=1,nvm
          IF( r_replant(ipts,ivm) .LT. nvm*2)THEN
             species_change_map(ipts,ivm)=NINT(r_replant(ipts,ivm))
          ELSE
             species_change_map(ipts,ivm)=0
          ENDIF
       ENDDO
    ENDDO

    r_replant(:,:) = val_exp
    var_name = 'fmc_map'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., r_replant(:,:), 'gather', nbp_glo, index_g)
    DO ipts=1,npts
       DO ivm=1,nvm
          IF( r_replant(ipts,ivm) .LT. nvm*2)THEN
             fm_change_map(ipts,ivm)=NINT(r_replant(ipts,ivm))
          ELSE
             fm_change_map(ipts,ivm)=0
          ENDIF
       ENDDO
    ENDDO

    ! The routines don't like to store logical variables.  So instead
    ! we create a real array.  If the value of the real array is one,
    ! we assign a value of TRUE to the logical array.  Anything
    ! else is .FALSE.
    r_replant(:,:) = val_exp
    var_name = 'lpft_replant'
    CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
         &        .TRUE., r_replant(:,:), 'gather', nbp_glo, index_g)
    DO ipts=1,npts
       DO ivm=1,nvm
          IF(ABS(r_replant(ipts,ivm) - un) .LT. 0.001)THEN
             lpft_replant(ipts,ivm)=.TRUE.
          ENDIF
       ENDDO
    ENDDO

   !-
    ! 19. Spinup
    !-
    IF (spinup_analytic) THEN

       IF (is_root_prc) THEN
          temp_global_years(1) = val_exp
          var_name = 'Global_years'
          CALL restget (rest_id_stomate, var_name, 1 ,1  , 1, itime, &
               &                .TRUE., temp_global_years)
          IF(temp_global_years(1) == val_exp) temp_global_years(1) = zero
          global_years = INT(temp_global_years(1))
       ENDIF
       CALL bcast(global_years)

       nbp_accu(:) = val_exp
       var_name = 'nbp_sum'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
            &              .TRUE., nbp_accu, 'gather', nbp_glo, index_g)
       IF (ALL(nbp_accu(:) == val_exp)) nbp_accu(:) = zero    

       nbp_flux(:) = val_exp
       var_name = 'nbp_flux'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, 1     , 1, itime, &
            &              .TRUE., nbp_flux, 'gather', nbp_glo, index_g)
       IF (ALL(nbp_flux(:) == val_exp)) nbp_flux(:) = zero     

       !-
       ok_equilibrium_real(:) = val_exp
       var_name = 'ok_equilibrium'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo , 1  , 1, itime, &
            &                .TRUE., ok_equilibrium_real,'gather', nbp_glo, index_g)
       IF (ALL(ok_equilibrium_real(:) == val_exp)) ok_equilibrium_real(:) = zero
       WHERE(ok_equilibrium_real(:) >= 0.5) 
          ok_equilibrium = .TRUE.
       ELSEWHERE
          ok_equilibrium = .FALSE.
       ENDWHERE

       MatrixV(:,:,:,:) = val_exp
       DO k = 1,nbpools
          DO j = 1,nbpools
             WRITE(part_str,'(I2)') k
             IF (k < 10) part_str(1:1) = '0'             
             var_name = 'MatrixV_'//part_str(1:LEN_TRIM(part_str))//'_'//TRIM(pools_str(j))
             CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm , 1, itime, &
                  &                     .TRUE., MatrixV(:,:,k,j), 'gather', nbp_glo, index_g)
          ENDDO
       ENDDO
       ! If nothing is found in the restart file, we initialize each submatrix by identity
       IF (ALL(MatrixV(:,:,:,:) == val_exp))  THEN 
          MatrixV(:,:,:,:) = zero
          DO l = 1,nbpools
             MatrixV(:,:,l,l) = un
          END DO
       END IF

       VectorU(:,:,:)  = val_exp
       DO k= 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0' 
          var_name = 'Vector_U_'//part_str(1:LEN_TRIM(part_str))
          CALL restget_p &
               &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &     .TRUE., VectorU(:,:,k), 'gather', nbp_glo, index_g)
          IF (ALL(VectorU(:,:,k) == val_exp))  VectorU(:,:,k) = zero
       ENDDO

       previous_stock(:,:,:)  = val_exp
       DO k= 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0' 
          var_name = 'previous_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restget_p &
               &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &     .TRUE., previous_stock(:,:,k), 'gather', nbp_glo, index_g)
          IF (ALL(previous_stock(:,:,k) == val_exp))  previous_stock(:,:,k) = undef_sechiba
       ENDDO

       current_stock(:,:,:)  = val_exp
       DO k= 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0' 
          var_name = 'current_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restget_p &
               &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &     .TRUE., current_stock(:,:,k), 'gather', nbp_glo, index_g)
          IF (ALL(current_stock(:,:,k) == val_exp))  current_stock(:,:,k) = zero
       ENDDO

    ENDIF ! spinup_matrix_method

    !-
    ! 20. Forest management.  Notice that if we're using functional allocation, we are also
    !    doing forestry in the sense that we have all the same information available.
    !-
    IF ( control%forest_management .OR. control%ok_functional_allocation ) THEN

       temp_real(:,:) = val_exp
       var_name = 'age_stand'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
       IF ( ALL(temp_real(:,:) == val_exp) ) THEN
          age_stand(:,:) = 0
       ELSE
          age_stand = NINT(temp_real)
       ENDIF

       mai(:,:) = val_exp
       var_name = 'mai'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., mai, 'gather', nbp_glo, index_g)
       IF ( ALL(mai(:,:) == val_exp) ) mai(:,:) = zero

       pai(:,:) = val_exp
       var_name = 'pai'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., pai, 'gather', nbp_glo, index_g)
       IF ( ALL(pai(:,:) == val_exp) ) pai(:,:) = zero

       previous_wood_volume(:,:) = val_exp
       var_name = 'previous_wood_volume'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., previous_wood_volume, 'gather', nbp_glo, index_g)
       IF ( ALL(previous_wood_volume(:,:) == val_exp) ) previous_wood_volume(:,:) = zero

       temp_real(:,:) = val_exp
       var_name = 'mai_count'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
       IF ( ALL(temp_real(:,:) == val_exp) ) THEN
          mai_count(:,:) = 0
       ELSE
          mai_count = NINT(temp_real)
       ENDIF

       coppice_dens(:,:) = val_exp
       var_name = 'coppice_dens'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., coppice_dens, 'gather', nbp_glo, index_g)
       IF ( ALL(coppice_dens(:,:) == val_exp) ) coppice_dens(:,:) = zero

       !-
       temp_real(:,:) = val_exp
       var_name = 'fm_lyear'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
       IF ( ALL(temp_real(:,:) == val_exp) ) THEN
          forest_managed_lastyear(:,:) = 0
       ELSE
          forest_managed_lastyear = NINT(temp_real)
       ENDIF

       IF (.NOT.lchange_species) THEN
          ! In the abscence of a lchange_species the model has to be
          ! driven by a prescribed value for forest_managed. This
          ! prescribed value can come from either a FMmap which is 
          ! read in or a value for ::FOREST_MANAGED_FORCED. In these 
          ! cases the values in the restart file are ignored.  
       ELSE
          ! We have a lchange_species which implies that we only want
          ! to read the FMmap once at the start of the simulation
          ! after which forest_managed becomes a prognostic variable 
          ! (as it may change after a clear cut) and therefore the 
          ! values from the maps are ignored and the value from the 
          ! restart is used except at the first time step.
          temp_real(:,:) = val_exp
          var_name = 'fm_current'
          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
               &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
          IF ( ALL(temp_real(:,:) == val_exp) ) THEN
             ! This is the first time step. The restart is still empty
             ! so use the values from the FMmap as read in stomate.f90
             ! Nothing should be done
          ELSE
             ! As soon as there is something in the restart file these
             ! values are used to overwrite the values that were read 
             ! from the FMmap.
             forest_managed = NINT(temp_real)
          ENDIF

       END IF
          
       !-
       !+++CHECK+++
       ! The comment is true for height but not for av_height
!!$       ! av_height is read just to show that it is in the restart file. It is not used 
!!$       ! and therefore it is not passed to stomate.f90. It is however written because to
!!$       ! restart file because slowproc.f90 checks wether it has a value. If it has a value
!!$       ! that value is used by sechiba. If not it implies that stomate.f90 was not used
!!$       ! and then it uses a prescribed height.
!!$       av_height(:,:) = val_exp
!!$       var_name = 'av_height'
!!$       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
!!$            &              .TRUE., av_height, 'gather', nbp_glo, index_g)
!!$       IF ( ALL(av_height(:,:) == val_exp) ) av_height(:,:) = un
       !+++++++++++
       !-  
       temp_real(:,:) = val_exp
       var_name = 'rotation_n'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
       IF ( ALL(temp_real(:,:) == val_exp) ) THEN
          rotation_n(:,:) = 1
       ELSE
          rotation_n(:,:) = NINT(temp_real(:,:))
       ENDIF
       !-
       temp_real(:,:) = val_exp
       var_name = 'last_cut'
       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
            &              .TRUE., temp_real, 'gather', nbp_glo, index_g)
       IF ( ALL(temp_real(:,:) == val_exp) ) THEN
          last_cut(:,:) = 0
       ELSE
          last_cut(:,:) = NINT(temp_real(:,:))
       ENDIF
       !-
!!$       lai_max_calc(:,:) = val_exp
!!$       var_name = 'lai_max_calc'
!!$       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
!!$            &              .TRUE., lai_max_calc, 'gather', nbp_glo, index_g)
!!$       IF ( ALL(lai_max_calc(:,:) == val_exp) ) lai_max_calc(:,:) = un
!!$       !-
!!$       biomass_lastyear(:,:,:,:) = val_exp
!!$       DO k = 1,nparts
!!$          WRITE(part_str,'(I2)') k
!!$          IF ( k < 10 ) part_str(1:1) = '0'
!!$          var_name = 'bm_lyear_'//part_str(1:LEN_TRIM(part_str))
!!$          CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
!!$               &                .TRUE., biomass_lastyear(:,:,k,icarbon), 'gather', nbp_glo, index_g)
!!$          IF ( ALL(biomass_lastyear(:,:,k,icarbon) == val_exp) ) biomass_lastyear(:,:,k,icarbon) = zero
!!$       ENDDO
!!$       !-
!!$       lai_max_lastyear(:,:) = val_exp
!!$       var_name = 'lai_max_lyear'
!!$       CALL restget_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
!!$            &              .TRUE., lai_max_lastyear, 'gather', nbp_glo, index_g)
!!$       IF ( ALL(lai_max_lastyear(:,:) == val_exp) ) lai_max_lastyear(:,:) = zero

    END IF ! (control%forest_management)

    IF (bavard >= 4) WRITE(numout,*) 'Leaving readrestart'

  END SUBROUTINE readrestart


!! ================================================================================================================================
!! SUBROUTINE   : writerestart
!!
!>\BRIEF         writes start file
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE writerestart (npts, index, day_counter, dt_days, &
       date, ind, adapted, regenerate, &
       moiavail_daily, gdd_init_date, litterhum_daily, t2m_daily, &
       t2m_min_daily, tsurf_daily, tsoil_daily, soilhum_daily, &
       precip_daily, gpp_daily, npp_daily, turnover_daily, &
       moiavail_month, moiavail_week, moiavail_growingseason, t2m_longterm, &
       tlong_ref, t2m_month, t2m_week, tsoil_month, &
       soilhum_month, fireindex, firelitter, maxmoiavail_lastyear, &
       maxmoiavail_thisyear, minmoiavail_lastyear, minmoiavail_thisyear, maxgppweek_lastyear, &
       maxgppweek_thisyear, gdd0_lastyear, gdd0_thisyear, precip_lastyear, &
       precip_thisyear, gdd_m5_dormance, gdd_from_growthinit, gdd_midwinter, &
       ncd_dormance, ngd_minus5, PFTpresent, npp_longterm, &
       lm_lastyearmax, lm_thisyearmax, maxfpc_lastyear, maxfpc_thisyear, &
       turnover_longterm, gpp_week, biomass, resp_maint_part, &
       leaf_age, leaf_frac, senescence, senescence_days, &
       when_growthinit, age, resp_hetero, resp_maint, &
       resp_growth, co2_fire, co2_to_bm_dgvm, veget_lastlight, &
       everywhere, need_adjacent, RIP_time, time_lowgpp, &
       time_hum_min, hum_min_dormance, litterpart, litter, &
       dead_leaves, carbon, black_carbon, lignin_struc, & 
       lignin_wood, turnover_time, prod_s, prod_m, prod_l, &
       flux_s, flux_m, flux_l, cflux_prod_s, cflux_prod_m, &
       cflux_prod_l, bm_to_litter, carb_mass_total, &
       age_stand, &
       rotation_n, last_cut, forest_managed, forest_managed_lastyear, &
       global_years, ok_equilibrium, nbp_accu, &
       nbp_flux, MatrixV, VectorU, previous_stock, &
       current_stock, rue_longterm, fpc_max, &
       circ_class_n, circ_class_biomass, KF, &
       mai, pai, previous_wood_volume, &
       mai_count, coppice_dens, vir_moiavail_growingseason, lai_per_level,&
       laieff_fit, temp_growth, wstress_season, wstress_month, &
       vir_wstress_fac, c_buried, k_latosa_adapt, species_change_map, &
       fm_change_map, lpft_replant)

    IMPLICIT NONE

    !! 0. Parameters and variables declaration

    !! 0.1 Input variables

    INTEGER(i_std), INTENT(in)                                         :: npts                    !! Domain size
    INTEGER(i_std), DIMENSION(npts), INTENT(in)                        :: index                   !! Indices of the points on the map
    REAL(r_std), INTENT(in)                                            :: day_counter             !! counts time until next STOMATE time step
    REAL(r_std), INTENT(in)                                            :: dt_days                 !! time step of STOMATE in days
    INTEGER(i_std), INTENT(in)                                         :: date                    !! date (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: ind                     !! density of individuals (1/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: adapted                 !! Winter too cold? between 0 and 1
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: regenerate              !! Winter sufficiently cold? between 0 and 1
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: moiavail_daily          !! daily moisture availability
    REAL(r_std), DIMENSION(npts,4), INTENT(in)                         :: gdd_init_date           !! date for beginning of gdd count
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: litterhum_daily         !! daily litter humidity
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: t2m_daily               !! daily 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: t2m_min_daily           !! daily minimum 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: tsurf_daily             !! daily surface temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                      :: tsoil_daily             !! daily soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                      :: soilhum_daily           !! daily soil humidity
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: precip_daily            !! daily precipitations (mm/day) (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: gpp_daily               !! daily gross primary productivity (gC/m**2/day)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: npp_daily               !! daily net primary productivity (gC/m**2/day)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in)      :: turnover_daily          !! daily turnover rates (gC/m**2/day)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: moiavail_month          !! "monthly" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: moiavail_week           !! "weekly" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: moiavail_growingseason  !! "growing season" moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: vir_moiavail_growingseason !! Virtual "growing season" moisture availability
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: t2m_longterm            !! "long term" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: tlong_ref               !! "long term" reference 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: t2m_month               !! "monthly" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: t2m_week                !! "weekly" 2 meter temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                      :: tsoil_month             !! "monthly" soil temperatures (K)
    REAL(r_std), DIMENSION(npts,nbdl), INTENT(in)                      :: soilhum_month           !! "monthly" soil humidity
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: fireindex               !! Probability of fire
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: firelitter              !! Longer term total litter above 
                                                                                                  !! the ground, gC/m**2 of ground
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: maxmoiavail_lastyear    !! last year's maximum 
                                                                                                  !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: maxmoiavail_thisyear    !! this year's maximum 
                                                                                                  !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: minmoiavail_lastyear    !! last year's minimum 
                                                                                                  !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: minmoiavail_thisyear    !! this year's minimum 
                                                                                                  !! moisture availability
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: maxgppweek_lastyear     !! last year's maximum weekly GPP
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: maxgppweek_thisyear     !! this year's maximum weekly GPP
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: gdd0_lastyear           !! last year's annual GDD0
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: gdd0_thisyear           !! this year's annual GDD0
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: precip_lastyear         !! last year's annual precipitation
                                                                                                  !! (mm/year)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: precip_thisyear         !! this year's annual precipitation 
                                                                                                  !! (mm/year)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: gdd_m5_dormance         !! growing degree days, threshold 
                                                                                                  !! -5 deg C (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: gdd_from_growthinit     !! growing degree days, 
                                                                                                  !! from begin of season
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: gdd_midwinter           !! growing degree days since 
                                                                                                  !! midwinter (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: ncd_dormance            !! number of chilling days since 
                                                                                                  !! leaves were lost (for phenology)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: ngd_minus5              !! number of growing days, threshold 
                                                                                                  !! -5 deg C (for phenology)
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                           :: PFTpresent              !! PFT exists (equivalent to 
                                                                                                  !! fpc_max > 0 for natural PFTs)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: npp_longterm            !! "long term" net primary productivity 
                                                                                                  !! (gC/m**2/year)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: lm_lastyearmax          !! last year's maximum leaf mass, 
                                                                                                  !! for each PFT (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: lm_thisyearmax          !! this year's maximum leaf mass, 
                                                                                                  !! for each PFT (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: maxfpc_lastyear         !! last year's maximum fpc for each 
                                                                                                  !! natural PFT, on ground
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: maxfpc_thisyear         !! this year's maximum fpc for each 
                                                                                                  !! PFT, on *total* ground (see stomate_season)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in)      :: turnover_longterm       !! "long term" turnover rate (gC/m**2/year)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: gpp_week                !! "weekly" GPP (gC/day/(m**2 covered)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in)      :: biomass                 !! biomass (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)                :: resp_maint_part         !! maintenance resp (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(in)             :: leaf_age                !! leaf age (days)
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(in)             :: leaf_frac               !! fraction of leaves in leaf age class
    REAL(r_std), DIMENSION(npts,nvm,ncirc), INTENT(in)                 :: circ_class_n            !! Number of trees in each circumference classes
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements), INTENT(in):: circ_class_biomass      !! Biomass components of the model tree within 
                                                                                                  !! each circumference classes (gC ind-1)
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                           :: senescence              !! is the plant senescent ? 
                                                                                                  !! (only for deciduous trees - 
                                                                                                  !! carbohydrate reserve)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: senescence_days         !! Days since start of senescence
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: when_growthinit         !! how many days ago was the beginning
                                                                                                  !! of the growing season
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: age                     !! mean age (years)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: resp_hetero             !! heterotrophic respiration (gC/day/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: resp_maint              !! maintenance respiration (gC/day/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: resp_growth             !! growth respiration (gC/day/m**2)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: co2_fire                !! carbon emitted into the atmosphere
                                                                                                  !! by fire (living and dead biomass)
                                                                                                  !! (in gC/m**2/time step)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: co2_to_bm_dgvm          !! biomass uptaken 
                                                                                                  !! (gC/(m**2 of total ground)/day)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: veget_lastlight         !! vegetation fractions (on ground) 
                                                                                                  !! after last light competition
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: everywhere              !! is the PFT everywhere in the 
                                                                                                  !! grid box or very localized 
                                                                                                  !! (after its introduction)
    LOGICAL, DIMENSION(npts,nvm), INTENT(in)                           :: need_adjacent           !! in order for this PFT to be introduced, 
                                                                                                  !! does it have to be present in an 
                                                                                                  !! adjacent grid box?
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: RIP_time                !! How much time ago was the PFT eliminated 
                                                                                                  !! for the last time (y)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: time_lowgpp             !! Duration of dormance (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: time_hum_min            !! time elapsed since strongest moisture
                                                                                                  !! availability (d)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: hum_min_dormance        !! minimum moisture during dormance
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(in)                 :: litterpart              !! fraction of litter above the ground
                                                                                                  !! belonging to different PFTs separated 
                                                                                                  !! for natural and agricultural PFTs.
    REAL(r_std), DIMENSION(npts,nlitt,nvm,nlevs,nelements), INTENT(in) :: litter                  !! metabolic and structural litter, 
                                                                                                  !! natural and agricultural, above and 
                                                                                                  !! below ground (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nlitt), INTENT(in)                 :: dead_leaves             !! dead leaves on ground, per PFT, 
                                                                                                  !! metabolic and structural, 
                                                                                                  !! in gC/(m**2 of ground)
    REAL(r_std), DIMENSION(npts,ncarb,nvm), INTENT(in)                 :: carbon                  !! carbon pool: active, slow, or passive,
                                                                                                  !! (gC/m**2)
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: black_carbon            !! black carbon on the ground 
                                                                                                  !! (gC/(m**2 of total ground))
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(in)                 :: lignin_struc            !! ratio Lignine/Carbon in structural litter,
                                                                                                  !! above and below ground, 
                                                                                                  !! (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(in)                 :: lignin_wood             !! ratio Lignine/Carbon in woody litter,
                                                                                                  !! above and below ground, 
                                                                                                  !! (gC/m**2)
    REAL(r_std), DIMENSION(npts,nvm,nparts), INTENT(in)                :: turnover_time           !!
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: age_stand               !! Age of stand (years)
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: rotation_n              !! Rotation number (number of rotation 
                                                                                                  !! since pft is managed)
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: last_cut                !! Years since last thinning (years)
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: forest_managed          !! forest management flag
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(in)                    :: forest_managed_lastyear !! forest management flag for the previous year
    REAL(r_std), DIMENSION(npts,0:nshort), INTENT(in)                  :: prod_s                  !!
    REAL(r_std), DIMENSION(npts,0:nmedium), INTENT(in)                 :: prod_m                  !!
    REAL(r_std), DIMENSION(npts,0:nlong), INTENT(in)                   :: prod_l                  !!
    REAL(r_std), DIMENSION(npts,nshort), INTENT(in)                    :: flux_s                  !!
    REAL(r_std), DIMENSION(npts,nmedium), INTENT(in)                   :: flux_m                  !!
    REAL(r_std), DIMENSION(npts,nlong), INTENT(in)                     :: flux_l                  !!
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: cflux_prod_s            !!
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: cflux_prod_m            !!
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: cflux_prod_l            !!
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), INTENT(in)      :: bm_to_litter            !!
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: carb_mass_total         !!
    INTEGER(i_std), INTENT(in)                                         :: global_years            !!
    LOGICAL, DIMENSION(npts), INTENT(in)                               :: ok_equilibrium          !!
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: nbp_accu                !! Accumulated Net Biospheric 
                                                                                                  !! Production over the year
    REAL(r_std), DIMENSION(npts), INTENT(in)                           :: nbp_flux                !! Net Biospheric Production over the year
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), INTENT(in)       :: MatrixV                 !!
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(in)               :: VectorU                 !!
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(in)               :: previous_stock          !! 
    REAL(r_std), DIMENSION(npts,nvm,nbpools), INTENT(in)               :: current_stock           !!
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: rue_longterm            !! Longterm radiation use efficiency
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: fpc_max                 !! Leaf projective cover  
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: KF                      !! Scaling factor to convert sapwood mass
                                                                                                  !! into leaf mass (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: k_latosa_adapt          !! Leaf to sapwood area adapted for water 
                                                                                                  !! stress. Adaptation takes place at the 
                                                                                                  !! end of the year (m)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: mai                     !! The mean annual increment
                                                                                                  !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: pai                     !! The period annual increment
                                                                                                  !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)                       :: previous_wood_volume    !! The volume of the tree trunks
                                                                                                  !! in a stand for the previous year.
                                                                                                  !! @tex $(m**3 / m**2 )$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(in)                     :: mai_count               !! The number of times we've
                                                                                                  !! calculated the volume increment
                                                                                                  !! for a stand
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)                        :: coppice_dens            !! The density of a coppice at the first
                                                                                                  !! cutting.
                                                                                                  !! @tex $( 1 / m**2 )$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm,nlevels_tot), INTENT(in)           :: lai_per_level           !! The amount of LAI in each physical
                                                                                                  !! canopy level.
                                                                                                  !! @tex $( m**2 / m**2 )$ @endtex 
    TYPE(laieff_type),DIMENSION (:,:,:),INTENT(in)                     :: laieff_fit              !! Fitted parameters for the effective LAI
    REAL(r_std),DIMENSION(npts),INTENT(in)                             :: temp_growth             !! Growth temperature (°C) 
                                                                                                  !! Is equal to t2m_month
    REAL(r_std), DIMENSION(:,:), INTENT(in)                            :: vir_wstress_fac         !! Virtual water stress factor, based on hum_rel_daily
                                                                                                  !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                            :: wstress_season          !! Water stress factor, based on hum_rel_daily
                                                                                                  !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                            :: wstress_month           !! Water stress factor, based on hum_rel_daily
                                                                                                  !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(in)                            :: c_buried                !! Carbon that is buried when land cover changes
                                                                                                  !! to a nobio type.  
                                                                                                  !! @tex $( gC )$ @endtex 
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)                         :: species_change_map      !! A map which gives the PFT number that each
                                                                                                  !! PFT will be replanted as in case of a clearcut.
                                                                                                  !! (1-nvm,unitless)
    INTEGER(i_std), DIMENSION(:,:), INTENT(in)                         :: fm_change_map           !! A map which gives the desired FM strategy when
                                                                                                  !! the PFT will be replanted after a clearcut.
                                                                                                  !! (1-nvm,unitless)
    LOGICAL, DIMENSION(:,:), INTENT(in)                                :: lpft_replant            !! Indicates if this PFT has either died this year
                                                                                                  !! or been clearcut/coppiced.  If it has, it is not
                                                                                                  !! replanted until the end of the year.

    !! 0.4 Local variables

    REAL(r_std)                                                        :: date_real               !! date, real
    REAL(r_std), DIMENSION(npts,nvm)                                   :: PFTpresent_real         !! PFT exists (equivalent to 
                                                                                                  !! fpc_max > 0 for natural PFTs), real
    REAL(r_std), DIMENSION(npts,nvm)                                   :: senescence_real         !! is the plant senescent ?
                                                                                                  !! (only for deciduous trees -
                                                                                                  !! carbohydrate reserve), real
    REAL(r_std), DIMENSION(npts,nvm)                                   :: need_adjacent_real      !! in order for this PFT to be introduced,
                                                                                                  !! does it have to be present in an 
                                                                                                  !! adjacent grid box? - real
    CHARACTER(LEN=80)                                                  :: var_name                !! To store variables names for I/O
    CHARACTER(LEN=10)                                                  :: part_str                !! string suffix indicating an index
    CHARACTER(LEN=10)                                                  :: part_str2               !! string suffix indicating an index
    CHARACTER(LEN=10)                                                  :: circ_str                !! string suffix indicating an index
    CHARACTER(LEN=3), DIMENSION(nlitt)                                 :: litter_str              !! string suffix indicating litter type
    CHARACTER(LEN=2), DIMENSION(nlevs)                                 :: level_str               !! string suffix indicating level
    REAL(r_std), DIMENSION(1)                                          :: xtmp                    !! temporary storage
    INTEGER(i_std)                                                     :: j,k,l,m,p,ibp           !! index
    INTEGER(i_std)                                                     :: idia, icut, ifm         !! index
    INTEGER(i_std)                                                     :: ipts, ivm               !! index
    CHARACTER(LEN=2), DIMENSION(nelements)                             :: element_str             !! string suffix indicating element
    REAL(r_std), DIMENSION(npts,nvm)                                   :: temp_real               !! temporary real to allow restget 
                                                                                                  !! to work on multi-dimensional integers
    REAL(r_std), DIMENSION(1)                                          :: temp_global_years       !!
    CHARACTER(LEN=7), DIMENSION(nbpools)                               :: pools_str               !!
    REAL(r_std), DIMENSION(npts)                                       :: ok_equilibrium_real     !!
    REAL(r_std), DIMENSION(npts,nvm)                                   :: lai                     !! Leaf area index 
                                                                                                  !! @tex $(m^{2}.m^{-2})$ @endtex 
    CHARACTER(LEN=80)                                                  :: cut_name                !! To store variables names for I/O
    CHARACTER(LEN=80)                                                  :: fm_name                 !! To store variables names for I/O
    REAL(r_std),DIMENSION(npts,nvm,nlevels_tot,nparams_laieff)         :: temp_array              !! To store structure values for I/O        
    INTEGER                                                            :: n,ilev                  !! Indices
    REAL(r_std), DIMENSION(npts,nvm)                                   :: r_replant               !! Writing integer and logical values
                                                                                                  !! to the restart file is not possible,
                                                                                                  !! so we convert to real first.
                                                                                                  !! 1.0 is TRUE for a logical array.
!_ ================================================================================================================================

    IF (bavard >= 3) WRITE(numout,*) 'Entering writerestart'
    !-
    ! 1 string definitions
    !-
    DO l=1,nlitt
       IF     (l == imetabolic) THEN
          litter_str(l) = 'met'
       ELSEIF (l == istructural) THEN
          litter_str(l) = 'str'
       ELSEIF (l == iwoody) THEN
          litter_str(l) = 'wod'
       ELSE
          CALL ipslerr_p (3,'stomate_io', &
               'Write restart - Define litter str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nlevs
       IF     (l == iabove) THEN
          level_str(l) = 'ab'
       ELSEIF (l == ibelow) THEN
          level_str(l) = 'be'
       ELSE
          CALL ipslerr_p (3,'stomate_io', &
               'Write restart - Define level str','','')
       ENDIF
    ENDDO
    !-
    DO l=1,nelements
       IF     (l == icarbon) THEN
          element_str(l) = ''
!!$       ELSEIF (l == initrogen) THEN
!!$          element_str(l) = '_n'
       ELSE
          CALL ipslerr_p (3,'stomate_io', &
               'Write restart - Define element str','','')
       ENDIF
    ENDDO
    !-
    pools_str(1:nbpools) =(/'str_ab ','str_be ','met_ab ','met_be ','wood_ab','wood_be', &
         'actif  ','slow   ','passif '/)
    !-
    IF (is_root_prc) THEN
       CALL ioconf_setatt_p ('UNITS','-')
       CALL ioconf_setatt_p ('LONG_NAME',' ')
    ENDIF
    !-
    ! 2 run control
    !-
    ! 2.1 day counter
    !-
    IF (is_root_prc) THEN
       var_name = 'day_counter'
       xtmp(1) = day_counter
       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, xtmp)
    ENDIF
    !-
    ! 2.2 time step of STOMATE in days
    !-
    IF (is_root_prc) THEN
       var_name = 'dt_days'
       xtmp(1) = dt_days
       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, xtmp)
    ENDIF
    !-
    ! 2.3 date
    !-
    IF (is_root_prc) THEN
       var_name = 'date'
       date_real = REAL(date,r_std)
       xtmp(1) = date_real
       CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, xtmp)
    ENDIF
    !-
    ! 3 daily meteorological variables
    !-
    var_name = 'moiavail_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_init_date'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    4, 1, itime, &
         &              gdd_init_date, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'litterhum_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                litterhum_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_min_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_min_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsurf_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                tsurf_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsoil_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                tsoil_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soilhum_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                soilhum_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'precip_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                precip_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'temp_growth'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                temp_growth, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'wstress_season'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                wstress_season, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'wstress_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                wstress_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'vir_wstress_fac'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                vir_wstress_fac, 'scatter', nbp_glo, index_g)
    var_name = 'c_buried'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nnobio, 1, itime, &
         &                c_buried, 'scatter', nbp_glo, index_g)
    
    !-
    ! 4 productivities
    !-
    var_name = 'gpp_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gpp_daily, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'npp_daily'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                npp_daily, 'scatter', nbp_glo, index_g)
    !-
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'turnover_daily_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                   turnover_daily(:,:,k,l), 'scatter', nbp_glo, index_g)
       ENDDO
    END DO
    !-
    ! 5 monthly meteorological variables
    !-
    var_name = 'moiavail_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'moiavail_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                moiavail_week, 'scatter', nbp_glo, index_g)
    !- 
    var_name = 'moiavail_gs' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, & 
         &              moiavail_growingseason, 'scatter', nbp_glo, index_g)
    !- 
    var_name = 'vir_moiavail_gs' 
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, & 
         &              vir_moiavail_growingseason, 'scatter', nbp_glo, index_g) 
    !-
    var_name = 't2m_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_longterm, 'scatter', nbp_glo, index_g)
    !-
    IF (control%ok_dgvm) THEN
       var_name = 'tlong_ref'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
            &                tlong_ref, 'scatter', nbp_glo, index_g)
    ENDIF
    !-
    var_name = 't2m_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 't2m_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                t2m_week, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'tsoil_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                tsoil_month, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'soilhum_month'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nbdl, 1, itime, &
         &                soilhum_month, 'scatter', nbp_glo, index_g)
    !-
    ! 6 fire probability
    !-
    var_name = 'fireindex'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                fireindex, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'firelitter'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                firelitter, 'scatter', nbp_glo, index_g)
    !-
    ! 7 maximum and minimum moisture availabilities for tropic phenology
    !-
    var_name = 'maxmoistr_last'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxmoiavail_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxmoistr_this'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxmoiavail_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'minmoistr_last'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                minmoiavail_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'minmoistr_this'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                minmoiavail_thisyear, 'scatter', nbp_glo, index_g)
    !-
    ! 8 maximum "weekly" GPP
    !-
    var_name = 'maxgppweek_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxgppweek_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxgppweek_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxgppweek_thisyear, 'scatter', nbp_glo, index_g)
    !-
    ! 9 annual GDD0
    !-
    var_name = 'gdd0_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                gdd0_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd0_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                gdd0_lastyear, 'scatter', nbp_glo, index_g)
    !-
    ! 10 annual precipitation
    !-
    var_name = 'precip_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                precip_thisyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'precip_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &                precip_lastyear, 'scatter', nbp_glo, index_g)
    !-
    ! 11 derived "biometeorological" variables
    !-
    var_name = 'gdd_m5_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gdd_m5_dormance, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_from_growthinit'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              gdd_from_growthinit, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'gdd_midwinter'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gdd_midwinter, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ncd_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ncd_dormance, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ngd_minus5'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ngd_minus5, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'time_lowgpp'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                time_lowgpp, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'time_hum_min'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                time_hum_min, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'hum_min_dormance'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                hum_min_dormance, 'scatter', nbp_glo, index_g)
    !-
    ! 12 Plant status
    !-
    var_name = 'PFTpresent'
    WHERE ( PFTpresent(:,:) )
       PFTpresent_real = un
    ELSEWHERE
       PFTpresent_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                PFTpresent_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'ind'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                ind, 'scatter', nbp_glo, index_g)
    !-
    DO k = 1,nparts
       WRITE(part_str,'(I2)') k
       IF (k < 10) part_str(1:1) = '0'
       var_name = 'turnover_time_'//part_str(1:LEN_TRIM(part_str))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                turnover_time(:,:,k), 'scatter', nbp_glo, index_g)
    ENDDO

    !-
    var_name = 'adapted'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                adapted, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'regenerate'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                regenerate, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'npp_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                npp_longterm, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lm_lastyearmax'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                lm_lastyearmax, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'lm_thisyearmax'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                lm_thisyearmax, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxfpc_lastyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxfpc_lastyear, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'maxfpc_thisyear'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                maxfpc_thisyear, 'scatter', nbp_glo, index_g)
    !-
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'turnover_longterm_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                   turnover_longterm(:,:,k,l), 'scatter', nbp_glo, index_g)
       ENDDO
    END DO
    !-
    var_name = 'gpp_week'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                gpp_week, 'scatter', nbp_glo, index_g)
    !-
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'biomass_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                   biomass(:,:,k,l), 'scatter', nbp_glo, index_g)
       ENDDO
    END DO
    !-
    DO ivm = 1,nvm
       DO ipts = 1,npts
          lai(ipts,ivm) = biomass_to_lai(biomass(ipts,ivm,ileaf,icarbon),ivm)
       ENDDO
    ENDDO
    var_name = 'lai'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                  lai, 'scatter', nbp_glo, index_g)
    !-
    DO k=1,nparts
       WRITE(part_str,'(I2)') k
       IF (k < 10) part_str(1:1) = '0'
       var_name = 'maint_resp_'//part_str(1:LEN_TRIM(part_str))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &                   resp_maint_part(:,:,k), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO m=1,nleafages
       WRITE(part_str,'(I2)') m
       IF (m < 10) part_str(1:1) = '0'
       var_name = 'leaf_age_'//part_str(1:LEN_TRIM(part_str))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &                  leaf_age(:,:,m), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO m=1,nleafages
       WRITE(part_str,'(I2)') m
       IF (m < 10) part_str(1:1) = '0'
       var_name = 'leaf_frac_'//part_str(1:LEN_TRIM(part_str))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &                   leaf_frac(:,:,m), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO m=1,nlevels_tot
       WRITE(part_str,'(I2)') m
       IF (m < 10) part_str(1:1) = '0'
       var_name = 'lai_per_level_'//part_str(1:LEN_TRIM(part_str))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &                   lai_per_level(:,:,m), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO ipts=1,npts
       DO ivm=1,nvm
          DO ilev=1,nlevels_tot
             temp_array(ipts,ivm,ilev,1)=laieff_fit(ipts,ivm,ilev)%a
             temp_array(ipts,ivm,ilev,2)=laieff_fit(ipts,ivm,ilev)%b
             temp_array(ipts,ivm,ilev,3)=laieff_fit(ipts,ivm,ilev)%c
             temp_array(ipts,ivm,ilev,4)=laieff_fit(ipts,ivm,ilev)%d
             temp_array(ipts,ivm,ilev,5)=laieff_fit(ipts,ivm,ilev)%e
          ENDDO
       ENDDO
    ENDDO
    DO m=1,nlevels_tot
       WRITE(part_str,'(I2)') m
       IF ( m < 10 ) part_str(1:1) = '0'
       DO n=1,nparams_laieff
          WRITE(part_str2,'(I2)') n
          IF ( n < 10 ) part_str2(1:1) = '0'

          var_name = 'laieff_fit__'//part_str(1:LEN_TRIM(part_str)) // '_' &
               // part_str2(1:LEN_TRIM(part_str2))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                   temp_array(:,:,m,n), 'scatter', nbp_glo, index_g)
       ENDDO
    ENDDO
    !
    DO m=1,ncirc
       WRITE(part_str,'(I2)') m
       IF (m < 10) part_str(1:1) = '0'
       var_name = 'circ_class_n_'//part_str(1:LEN_TRIM(part_str))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &                   circ_class_n(:,:,m), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO m = 1,nelements
       DO l = 1,nparts
          DO k = 1,ncirc 
             WRITE(part_str,'(I2)') l
             IF ( l < 10 ) part_str(1:1) = '0'
             WRITE(circ_str, '(I2.2)') k
             var_name = 'cc_bio_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(m))//'_'//TRIM(circ_str)
             CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
                &         circ_class_biomass(:,:,k,l,m), 'scatter', nbp_glo, index_g)
          ENDDO
       ENDDO
    ENDDO     
    !-
    var_name = 'senescence'
    WHERE ( senescence(:,:) )

       senescence_real = un

    ELSEWHERE

       senescence_real = zero

    ENDWHERE

    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                senescence_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'senescence_days'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              senescence_days, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'when_growthinit'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                when_growthinit, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'age'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm  , 1, itime, &
         &                age, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'KF'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
          &              KF(:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'k_latosa_adapt'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
          &              k_latosa_adapt(:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'rue_longterm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
          &              rue_longterm(:,:), 'scatter', nbp_glo, index_g)
    !-
!!$    var_name = 'lai_target'
!!$    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
!!$          &              lai_target(:,:), 'scatter', nbp_glo, index_g)
    !-
    var_name = 'fpc_max'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
          &              fpc_max(:,:), 'scatter', nbp_glo, index_g)
    !-
    ! 13 CO2
    !-
    var_name = 'resp_hetero'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_hetero, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'resp_maint'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_maint, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'resp_growth'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                resp_growth, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'co2_fire'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, 1, itime, &
         &                co2_fire, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'co2_to_bm_dgvm'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                co2_to_bm_dgvm, 'scatter', nbp_glo, index_g)
    !-
    ! 14 vegetation distribution after last light competition
    !-
    var_name = 'veget_lastlight'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                veget_lastlight, 'scatter', nbp_glo, index_g)
    !-
    ! 15 establishment criteria
    !-
    var_name = 'everywhere'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                everywhere, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'need_adjacent'
    WHERE (need_adjacent(:,:))
       need_adjacent_real = un
    ELSEWHERE
       need_adjacent_real = zero
    ENDWHERE
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                need_adjacent_real, 'scatter', nbp_glo, index_g)
    !-
    var_name = 'RIP_time'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &                RIP_time, 'scatter', nbp_glo, index_g)
    !-
    ! 16 black carbon
    !-
    var_name = 'black_carbon'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo,    1, 1, itime, &
         &                black_carbon, 'scatter', nbp_glo, index_g)
    !-
    ! 17 litter
    !-
    DO l=1,nlitt
       var_name = 'litterpart_'//litter_str(l)
       CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, 1, itime, &
            &                   litterpart(:,:,l), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO k = 1,nelements
       DO l = 1,nlevs
          DO m = 1,nvm
             WRITE (part_str, '(I2)') m
             IF (m<10) part_str(1:1)='0'
             var_name = 'litter_'//part_str(1:LEN_TRIM(part_str))//'_'//level_str(l)//TRIM(element_str(k))
             CALL restput_p (rest_id_stomate, var_name, nbp_glo, nlitt, 1, itime, &
                  &                     litter(:,:,m,l,k), 'scatter', nbp_glo, index_g)
          ENDDO
       ENDDO
    END DO
    !-
    DO l=1,nlitt
       var_name = 'dead_leaves_'//litter_str(l)
       CALL restput_p (rest_id_stomate, var_name, nbp_glo,  nvm, 1, itime, &
            &                   dead_leaves(:,:,l), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO m=1,nvm
       WRITE (part_str, '(I2)') m
       IF (m<10) part_str(1:1)='0'
       var_name = 'carbon_'//part_str(1:LEN_TRIM(part_str))
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, ncarb, 1, itime, &
            &                   carbon(:,:,m), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO l=1,nlevs
       var_name = 'lignin_struc_'//level_str(l)
       CALL restput_p &
            &      (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &       lignin_struc(:,:,l), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    DO l=1,nlevs
       var_name = 'lignin_wood_'//level_str(l)
       CALL restput_p &
            &    (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &     lignin_wood(:,:,l), 'scatter', nbp_glo, index_g)
    ENDDO
    !-
    ! 18 land cover change
    !-
    var_name = 'prod_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nshort+1, 1, itime, &
         &                prod_s, 'scatter', nbp_glo, index_g)
    var_name = 'prod_m'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nmedium+1, 1, itime, &
         &                prod_m, 'scatter', nbp_glo, index_g)
    var_name = 'prod_l'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nlong+1, 1, itime, &
         &                prod_l, 'scatter', nbp_glo, index_g)
    var_name = 'flux_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nshort, 1, itime, &
         &                flux_s, 'scatter', nbp_glo, index_g)
    var_name = 'flux_m'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nmedium, 1, itime, &
         &                flux_m, 'scatter', nbp_glo, index_g)
    var_name = 'flux_l'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nlong, 1, itime, &
         &                flux_l, 'scatter', nbp_glo, index_g)
    var_name = 'cflux_prod_s'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &              cflux_prod_s, 'scatter', nbp_glo, index_g)
    var_name = 'cflux_prod_m'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &              cflux_prod_m, 'scatter', nbp_glo, index_g)
    var_name = 'cflux_prod_l'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &              cflux_prod_l, 'scatter', nbp_glo, index_g)
    DO l = 1,nelements
       DO k = 1,nparts
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0'
          var_name = 'bm_to_litter_'//part_str(1:LEN_TRIM(part_str))//TRIM(element_str(l))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                bm_to_litter(:,:,k,l), 'scatter', nbp_glo, index_g)
       ENDDO
    END DO

    var_name = 'carb_mass_total'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
         &              carb_mass_total, 'scatter', nbp_glo, index_g)

    ! For these next three variables, we have to convert to a real array first.
    r_replant(:,:)=REAL(species_change_map(:,:),r_std)
    var_name = 'sc_map'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              r_replant, 'scatter', nbp_glo, index_g)
    
    r_replant(:,:)=REAL(fm_change_map(:,:),r_std)
    var_name = 'fmc_map'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              r_replant, 'scatter', nbp_glo, index_g)

    DO ipts=1,npts
       DO ivm=1,nvm
          IF(lpft_replant(ipts,ivm))THEN
             r_replant(ipts,ivm)=un
          ELSE
             r_replant(ipts,ivm)=zero
          ENDIF
       END DO
    END DO
    var_name = 'lpft_replant'
    CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
         &              r_replant, 'scatter', nbp_glo, index_g)
    !-
    ! 19. Spinup
    !-
    IF (spinup_analytic) THEN

       IF (is_root_prc) THEN
          temp_global_years(1) = REAL(global_years)
          var_name='Global_years'
          CALL restput (rest_id_stomate, var_name, 1, 1, 1, itime, temp_global_years)
       ENDIF
       
       var_name = 'nbp_sum'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &              nbp_accu, 'scatter', nbp_glo, index_g)

       var_name = 'nbp_flux'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &              nbp_flux, 'scatter', nbp_glo, index_g)

       var_name = 'ok_equilibrium'
       WHERE(ok_equilibrium(:))
          ok_equilibrium_real = un
       ELSEWHERE
          ok_equilibrium_real = zero
       ENDWHERE
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, 1, 1, itime, &
            &               ok_equilibrium_real, 'scatter', nbp_glo, index_g)
       
       DO k = 1,nbpools
          DO j = 1,nbpools
             WRITE(part_str,'(I2)') k
             IF (k < 10) part_str(1:1) = '0'             
             var_name = 'MatrixV_'//part_str(1:LEN_TRIM(part_str))//'_'//TRIM(pools_str(j))
             CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
                  &                MatrixV(:,:,k,j), 'scatter', nbp_glo, index_g)
          ENDDO
       ENDDO
          
       DO k = 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0' 
          var_name = 'Vector_U_'//part_str(1:LEN_TRIM(part_str))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                VectorU(:,:,k), 'scatter', nbp_glo, index_g)
       ENDDO
          
       DO k = 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0' 
          var_name = 'previous_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                previous_stock(:,:,k), 'scatter', nbp_glo, index_g)
       ENDDO
          
       DO k = 1,nbpools
          WRITE(part_str,'(I2)') k
          IF (k < 10) part_str(1:1) = '0' 
          var_name = 'current_stock_'//part_str(1:LEN_TRIM(part_str))
          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
               &                current_stock(:,:,k), 'scatter', nbp_glo, index_g)
       ENDDO
       
    ENDIF !(spinup_analytic)

    !-
    ! 20. Forest management
    !-
    IF ( control%forest_management .OR. control%ok_functional_allocation) THEN

!!$       var_name = 'lai_max_calc'
!!$       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
!!$            &              lai_max_calc, 'scatter', nbp_glo, index_g)
!!$
!!$       DO k = 1,nparts
!!$          WRITE(part_str,'(I2)') k
!!$          IF (k < 10) part_str(1:1) = '0'
!!$          var_name = 'bm_lyear_'//part_str(1:LEN_TRIM(part_str))
!!$          CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
!!$               &                biomass_lastyear(:,:,k,icarbon), 'scatter', nbp_glo, index_g)
!!$       ENDDO
!!$
!!$       var_name = 'lai_max_lyear'
!!$       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
!!$            &              lai_max_lastyear, 'scatter', nbp_glo, index_g)

       var_name = 'mai'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              mai, 'scatter', nbp_glo, index_g)

       var_name = 'pai'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              pai, 'scatter', nbp_glo, index_g)

       var_name = 'coppice_dens'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              coppice_dens, 'scatter', nbp_glo, index_g)

       var_name = 'previous_wood_volume'
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              previous_wood_volume, 'scatter', nbp_glo, index_g)

       var_name = 'age_stand'
       temp_real = REAL(age_stand,r_std)
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              temp_real, 'scatter', nbp_glo, index_g)

       var_name = 'rotation_n'
       temp_real = REAL(rotation_n,r_std)  
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              temp_real, 'scatter', nbp_glo, index_g)

       var_name = 'last_cut'
       temp_real = REAL(last_cut,r_std)
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              temp_real, 'scatter', nbp_glo, index_g)

       var_name = 'fm_lyear'
       temp_real = REAL(forest_managed_lastyear,r_std)
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              temp_real, 'scatter', nbp_glo, index_g)

       var_name = 'fm_current'
       temp_real = REAL(forest_managed,r_std)
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              temp_real, 'scatter', nbp_glo, index_g)
       
       var_name = 'mai_count'
       temp_real = REAL(mai_count,r_std)
       CALL restput_p (rest_id_stomate, var_name, nbp_glo, nvm, 1, itime, &
            &              temp_real, 'scatter', nbp_glo, index_g)

    END IF ! control%forest_management

    IF (bavard >= 4) WRITE(numout,*) 'Leaving writerestart'

  END SUBROUTINE writerestart

!! ================================================================================================================================
!! SUBROUTINE   : readbc
!!
!>\BRIEF         
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S):
!!
!! REFERENCE(S) : None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE readbc (npts, lalo, neighbours, resolution, contfrac, tref)

    IMPLICIT NONE

    !! 0. Parameters and variables declaration
    
    !! 0.1 Input variables  
    
    INTEGER(i_std), INTENT(in)                  :: npts        !! Domain size
    REAL(r_std), DIMENSION(npts,2), INTENT(in)  :: lalo        !! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(r_std), DIMENSION(npts,2), INTENT(in)  :: resolution  !! size in x an y of the grid (m)
    INTEGER(i_std),DIMENSION (npts,8), INTENT(in):: neighbours !! Neighbouring land grid cell
    REAL(r_std),DIMENSION (npts), INTENT(in)     :: contfrac   !! Fraction of land in each grid box 


    !! 0.3 Modified variables

    REAL(r_std), DIMENSION(npts), INTENT(inout) :: tref        !! "long term" reference 2 meter temperatures (K)

!_ ================================================================================================================================

    !! If the vegetation is static, then the long-term reference
    !! temperature is a boundary condition.

    IF ( .NOT. control%ok_dgvm ) THEN
       CALL get_reftemp (npts, lalo, neighbours, resolution, contfrac, tref )
    ENDIF

  END SUBROUTINE readbc
  !-
  !===
  !-
  SUBROUTINE get_reftemp_clear
    !---------------------------------------------------------------------
    firstcall=.TRUE.
    IF (ALLOCATED (trefe)) DEALLOCATE( trefe )
    !-------------------------------
  END SUBROUTINE get_reftemp_clear
  !-
  !===
  !-
  SUBROUTINE get_reftemp (npts, lalo, neighbours, resolution, contfrac, tref_out)
    !---------------------------------------------------------------------
    !- read the long-term reference temperature from a boundary condition
    !- file. If the vegetation is dynamic, this field is used to
    !- initialize correctly the (prognostic) long-term reference
    !- temperature (in the case it is not found in the restart file).
    !- If the vegetation is static, the field read here is a real boundary
    !- condition that is not modified by the model.
    !---------------------------------------------------------------------
    !-
    ! 0 declarations
    !-
    ! 0.1 input
    !-
    ! Domain size
    INTEGER(i_std),INTENT(in)                      :: npts        !! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(r_std),DIMENSION (npts,2),INTENT(in)      :: lalo        !! size in x an y of the grid (m)
    REAL(r_std),DIMENSION (npts,2),INTENT(in)      :: resolution  !! Vector of latitude and longitudes (degrees)
    INTEGER(i_std), DIMENSION (npts,8), INTENT(in) :: neighbours  !! Vector of neighbours for each grid point 
    REAL(r_std), DIMENSION (npts), INTENT(in)      :: contfrac    !! Fraction of land in each grid cell (unitless)   

    !-
    ! 0.2 output
    !-
    ! reference temperature (K)
    REAL(r_std), DIMENSION(npts),INTENT(out)       :: tref_out    !! interpolated reference temperature
    !-
    ! 0.3 local
    !-
    INTEGER(i_std)                                 :: nbvmax      !! nbvmax for interpolation (unitless). It is the 
    CHARACTER(LEN=80)                              :: filename    !! filename of temperature file
    INTEGER(i_std)                                :: iml, jml, lml, &
              & tml, fid, ib, ip, jp, fopt, ilf, lastjp, nbexp,ic      !! Indices
    REAL(r_std)                                   :: lev(1), date, dt      !! Help variables to read in file data
    INTEGER(i_std)                                :: itau(1)               !! Help variables to read in file data
    REAL(r_std)                                   :: sgn                   !! Help variable to normalise temp.
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)      :: lat_rel               !! Help variable to read file data and allocate memory
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)      :: lon_rel               !! Help variable to read file data and allocate memory
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)      :: tref_file             !! Help variable to read file data and allocate memory
    REAL(r_std), ALLOCATABLE, DIMENSION(:,:)      :: sub_area              !! Help variable to read file data and allocate memory
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:,:) :: sub_index             !! Help variable to read file data and allocate memory
    INTEGER(i_std), ALLOCATABLE, DIMENSION(:,:)   :: mask                  !! Help variable to read file data and allocate memory
    CHARACTER(LEN=30)                             :: callsign              !! Help variable to read file data and allocate memory
    LOGICAL                                       :: ok_interpol = .FALSE. !! Optional return of aggregate_2d
    INTEGER                                       :: ALLOC_ERR             !! Help varialbe to count allocation error
!_ ================================================================================================================================
    !-
    ! If this is the first call, calculate the reference temperature
    ! and keep it in memory
    !-
    IF (firstcall) THEN
       !---
       !-- 1.1 only do this once
       !---
       firstcall = .FALSE.
       !---
       !-- 1.2 allocate the field
        !-
         ! Allocate memory for temperature
         ALLOC_ERR=-1
         ALLOCATE(trefe(npts), STAT=ALLOC_ERR)
         IF (ALLOC_ERR/=0) THEN
            WRITE(numout,*) "ERROR IN ALLOCATION of trefe : ",ALLOC_ERR
            CALL ipslerr_p (3,'stomate_io', 'get_reftemp','','') 
         ENDIF
         !-
        trefe(:) = zero

        !-
        tref_out(:) = zero
 

       ! - 
       !---
       !-- 1.3 read and interpolate the temperature file
       !---
       !-- Needs to be a configurable variable
       !---
       !Config Key   = REFTEMP_FILE
       !Config Desc  = Name of file from which the reference temperature is read
       !Config If    = OK_STOMATE
       !Config Def   = reftemp.nc
       !Config Help  = The name of the file to be opened to read
       !Config         the reference surface temperature.
       !Config         The data from this file is then interpolated
       !Config         to the grid of of the model.
       !Config         The aim is to get a reference temperature either
       !Config         to initialize the corresponding prognostic model
       !Config         variable correctly (ok_dgvm=TRUE) or to impose it
       !Config         as boundary condition (ok_dgvm=FALSE)
       !Config Units = [FILE]
       !---
       filename = 'reftemp.nc'
       CALL getin_p('REFTEMP_FILE',filename)
       !---
       IF (is_root_prc) CALL flininfo(filename,iml, jml, lml, tml, fid)
       CALL bcast(iml)
       CALL bcast(jml)
       CALL bcast(lml)
       CALL bcast(tml)
       !---
       ! Allocate memory for latitudes
       ALLOC_ERR=-1
       ALLOCATE(lat_rel(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of lat_rel : ",ALLOC_ERR
          CALL ipslerr_p (3,'stomate_io', 'get_reftemp','','')
       ENDIF
       !-
       lat_rel(:,:) = zero
       !-
       ! Allocate memory for longitude
       ALLOC_ERR=-1
       ALLOCATE(lon_rel(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of lon_rel : ",ALLOC_ERR
          CALL ipslerr_p (3,'stomate_io', 'get_reftemp','','')
       ENDIF
       !-
       lon_rel(:,:) = zero
       !-
       ! Allocate memory for mask
       ALLOC_ERR=-1
       ALLOCATE(mask(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of mask : ",ALLOC_ERR
          CALL ipslerr_p (3,'stomate_io', 'get_reftemp','','')
       ENDIF
       !-
       mask(:,:) = zero
       !-
       ! Allocate memory for temperature data
       ALLOC_ERR=-1
       ALLOCATE(tref_file(iml,jml), STAT=ALLOC_ERR)
       IF (ALLOC_ERR/=0) THEN
          WRITE(numout,*) "ERROR IN ALLOCATION of tref_file : ",ALLOC_ERR
          CALL ipslerr_p (3,'stomate_io', 'get_reftemp','','')
       ENDIF
       !-
       tref_file(:,:) = zero
       !-
       IF (is_root_prc) CALL flinopen (filename, .FALSE., iml, jml, lml, &
            &                                   lon_rel, lat_rel, lev, tml, itau, date, dt, fid)
       CALL bcast(lon_rel)
       CALL bcast(lat_rel)
       CALL bcast(lev)
       CALL bcast(itau)
       CALL bcast(date)
       CALL bcast(dt)
       !-
       IF (is_root_prc) CALL flinget (fid, 'temperature', iml, jml, lml, tml, &
            &                                  1, 1, tref_file)
       CALL bcast(tref_file)
       !-
       IF (is_root_prc) CALL flinclo (fid)
       !-
       ! Create mask with values of temperature
       mask(:,:) = zero
       DO ip=1,iml
          DO jp=1,jml
             IF (tref_file(ip,jp) > min_sechiba) THEN
                mask(ip,jp) = un
             ENDIF
          ENDDO
       ENDDO
       !---
       ! Set nbvmax to 200 for interpolation
       ! This number is the dimension of the variables in which we store 
       ! the list of points of the source grid which fit into one grid box of the target. 
       nbvmax = 200
       !---
       callsign = 'Reference temperature'
       !---
       ! Start with interpolation
       DO WHILE ( .NOT. ok_interpol )
          WRITE(numout,*) "Projection arrays for ",callsign," : "
          WRITE(numout,*) "nbvmax = ",nbvmax
          !---
          ALLOC_ERR=-1
          ALLOCATE(sub_area(npts,nbvmax), STAT=ALLOC_ERR)
          IF (ALLOC_ERR/=0) THEN
             WRITE(numout,*) "ERROR IN ALLOCATION of sub_area : ",ALLOC_ERR
             CALL ipslerr_p (3,'stomate_io', 'get_reftemp','','')
          ENDIF
          sub_area(:,:)=zero
          ALLOC_ERR=-1
          ALLOCATE(sub_index(npts,nbvmax,2), STAT=ALLOC_ERR)
          IF (ALLOC_ERR/=0) THEN
             WRITE(numout,*) "ERROR IN ALLOCATION of sub_index : ",ALLOC_ERR
             CALL ipslerr_p (3,'stomate_io', 'get_reftemp','','') 
          ENDIF
          sub_index(:,:,:)=0
          !-
          WRITE(numout,*) "Start interpolation in STOMATE_IO: get_reftemp"
          !-
          CALL aggregate_p(npts, lalo, neighbours, resolution, contfrac, &
               &                iml, jml, lon_rel, lat_rel, mask, callsign, &
               &                nbvmax, sub_index, sub_area, ok_interpol)
          !-
          IF ( .NOT. ok_interpol ) THEN
             DEALLOCATE(sub_area)
             DEALLOCATE(sub_index)
          ENDIF
          !-
          nbvmax = nbvmax * 2
       ENDDO
       !-
       ! Check how many points are found
       !-
       nbexp = zero
       trefe(:) = zero
       !-       
       DO ib=1,npts ! Loop over domain size
          !-
          fopt =  COUNT(sub_area(ib,:) > zero)
          !-
          !! Compute the temperature parameters
          !-
          IF ( fopt .GT. 0) THEN      ! If no points were interpolated
             !-
             sgn = zero
             !-
             DO ilf = 1,fopt         ! If points were interpolated
                !- 
                ip = sub_index(ib,ilf,1)
                jp = sub_index(ib,ilf,2)
                !-
                ! Weighted values by interpolation area
                trefe(ib) = trefe(ib) + tref_file(ip,jp) * sub_area(ib,ilf)
                sgn = sgn + sub_area(ib,ilf)
                !-
             ENDDO
             !-
             ! Normalize the surface
             IF ( sgn .LT. min_sechiba) THEN
                nbexp = nbexp + 1
             ELSE
                trefe(ib) = trefe(ib)/sgn         
             ENDIF
             !-
          ELSE
             !-
             nbexp = nbexp + 1
             !-
             !
!jot ++++++++++++++TEMP
!    Here we set all pixels that didn't have enough surrounding pixels
!    for the interpolation to the mean average 2m temperature. This
!    is not ideal. 
!    The ideal case: here the pixels get the value 'undef' and in 
!    a next step these pixels get the value from a surrounding pixel
!    that has an interpolated value.
!    We tried this, see the following loop. We used the 'neighbours'
!    variable calculated in grid.f90 to check if one of the eight 
!    surrounding pixels has a temperature value. 
!    However, this doesn't work. The dimensions of refe(neighbours(ib,ic)
!    aren't correct. Maybe something goes wrong with the 
!    parallelisation? Needs to be checked.
!jot ++++++++++++++TEMP  
             ! 
             trefe(ib) = mstemp
!jot             trefe(ib) = undef
             !- 
          ENDIF
          !-
       ENDDO
       !- 
       !
       IF ( nbexp .GT. 0 ) THEN
          WRITE(numout,*) 'STOMATE_IO, get_reftemp : The interpolation of the temperature had ', nbexp
          WRITE(numout,*) 'STOMATE_IO, get_reftemp : points without data out of total', SIZE(trefe)
          WRITE(numout,*) 'STOMATE_IO, get_reftemp : interpolation points.' 
          WRITE(numout,*) 'STOMATE_IO, get_reftemp : We will start now with replacing the pixel'
          WRITE(numout,*) 'STOMATE_IO, get_reftemp : by the value of the nearest neighbour'
       ENDIF
       !- 
       ! look for values that have the value undef to replace them by the value 
       ! of the nearest neighbour
       ! -
       DO ib=1,npts ! Loop over domain size
          !-
          ! check if this point is one without a temperature value
          !-
          IF (trefe(ib) .EQ. undef) THEN
              !-
              ! in the module grid.f90 the variable neighbours is calculated
              ! it tests the 8 neighbour pixels, a value of -1 means that this
              ! is not a land point or outside of the domain, here we loop over
              ! these eight pixels to see if we find a pixel with a temperature
              ! value 
              !- 
              DO ic=1,8 ! number of neighbour pixels
                 !-
                 ! check if there is a neighbour land pixel
                 !-
                 IF (neighbours(ib,ic) .GT. 0 ) THEN
                    !-
                    ! now we have neighbouring land pixel but 
                    ! check if his pixel contains a temperature
                    ! value and that this not an 'artifial' value
                    !-     
                    IF (trefe(neighbours(ib,ic)) .NE. undef .AND. &
                        trefe(neighbours(ib,ic)) .NE. mstemp ) THEN
                        !-
                        ! set the temperature value
                        !- 
                        trefe(ib) =  trefe(neighbours(ib,ic))
                        !-
                        ! cool - we replaced the pixel having no 
                        ! temperature value with the temperature value
                        ! of a neighbouring land pixel, continue checking
                        ! the following pixels
                        !-
                        EXIT
                        !-
                    ENDIF 
                    !-
                    ! when we tested the last neighbouring pixel and
                    ! still haven't found a pixel with a temperatue value
                    ! then replace it with the global annual mean surface
                    ! temperature value
                    !-                
                 ELSEIF (ic .EQ. 8) THEN ! if all neighbours are checked
                    !- 
                    trefe(ib) = mstemp
                    !-
                 ENDIF
                 !-
                 ! 
             ENDDO ! neighbouring pixels  
             !-
          ENDIF
          !-
          !
       ENDDO !domain size
       !-
       ! transform into Kelvin
       !-
       trefe(:) = trefe(:) + ZeroCelsius
       !-
       ! deallocate
       !-                       
       DEALLOCATE (lat_rel)
       DEALLOCATE (lon_rel)
       !  
    ENDIF
    !-
    ! output the reference temperaturei
    !-
    tref_out(:) = trefe(:)
    !-
    WRITE(numout,*) 'STOMATE_IO, get_reftemp : Done'
    !-
    !
  END SUBROUTINE get_reftemp
  !      
  !-
END MODULE stomate_io
